require 'puppet/provider/equallogic'

Puppet::Type.type(:equallogic_volume_clone).provide(:equallogic_volume_clone, :parent => Puppet::Provider::Equallogic) do
  @doc = "Manage Equallogic Volume cloning."

  confine :feature => :posix
  defaultfor :feature => :posix
  $provider_path = Pathname.new(__FILE__).parent
  $pythonFilePath = "#{$provider_path}/../../util/network_device/equallogic/pythonEquallogic.py"
  $volumeFilePath = "#{$provider_path}/../../util/network_device/equallogic/volume"
  mk_resource_methods
  $is_new_vol_exist = "false"
  # Volume info getter
  def get_volinfo
    response = `"#{$volumeFilePath}" show `
    volume_info = []
    counter = 0
    volcount = 1
    response.split("\n").each do |line|
      if (counter < 3)
        counter = counter + 1
        next
      else
        volume_info << create_vol_info(line)
        volcount = volcount + 1
      end
    end
    Puppet.debug("Processed all volumes. Returning info array.")
    # Return volume_info array
    volume_info
  end

  def create_vol_info(line)
    array = line.split(' ')
    vol_name = array[0]
    Puppet.debug("Puppet::Provider::Equallogic_volume_clone get_volinfo: Processing volume #{vol_name}.")
    # Construct hash
    vol_info = { :name         => vol_name
    }
    return vol_info
  end
  
  def create_volume_clone
    response = ` python #{$pythonFilePath} #{transport.host}  #{transport.user}  #{transport.password} volumeclone #{@resource[:volume_name]}  #{@resource[:name]} `
    return response
  end

  # Volume clone.
  def create
    Puppet.info("Puppet::Provider::Equallogic_volume_clone: Cloning Equallogic Volume #{@resource[:volume_name]} by the name of #{@resource[:name]}.")
    # Call python api to create volume.
    response = create_volume_clone
    if (response.match("% Error") || response.match("Error: Only ASCII"))
      raise Puppet::Error, "Unable to clone the volume  #{@resource[:name]} because of the following exception: #{response}."
    end
    Puppet.info("Puppet::Provider::Equallogic_volume_clone: Volume #{@resource[:name]} created successfully.")
    Puppet.debug("Puppet::Provider::Equallogic_volume_clone: Volume creation response :: #{response}")
    return true
  end

  def destroy
    Puppet.info("Puppet::Provider::Equallogic_volume_clone: To delete equallogic volume, use volume delete process.")
    @property_hash[:ensure] = :absent
  end

  def exists?
    Puppet.info("Puppet::Provider::Equallogic_volume_clone: checking existence of Equallogic Volume #{@resource[:name]}")
    volume_info = get_volinfo
    volume_info.each do |volume|
      # Resource existence validation
      Puppet.debug("Puppet::Provider::Equallogic_volume_clone_ self.instances: Comparing if resource already exists at equallogic...")
      if("#{@resource[:name]}" == volume[:name])
        Puppet.info("Puppet::Provider::Equallogic_volume_clone_ self.instances: Resource name #{@resource[:name]} already exists.")
        $is_new_vol_exist = "true"
      end
    end
    if("#{@resource[:ensure]}" == "present" && $is_new_vol_exist == "true")
      Puppet.debug("volume already exist...")
      @property_hash[:ensure] = :present
    else
      Puppet.debug("volume does not exist.$is_new_vol_exist Hence creating one...")
      @property_hash[:ensure] = :absent
    end
    Puppet.debug("Value = #{@property_hash[:ensure]}")
    @property_hash[:ensure] == :present
  end
end
