require 'puppet/util/network_device'
require 'puppet/util/network_device/equallogic/facts'
require 'puppet/util/network_device/transport_equallogic'
require 'uri'
require 'net/https'
require '/etc/puppetlabs/puppet/modules/asm_lib/lib/security/encode'


class Puppet::Util::NetworkDevice::Equallogic::Device

  attr_accessor :url, :transport

  $provider_path = Pathname.new(__FILE__).parent
  $pythonFilePath = "#{$provider_path}/pythonEquallogic.py"
  def initialize(url, option = {})
    Puppet.debug("Device login started")
    @url = URI.parse(url)
    redacted_url = @url.dup
    redacted_url.password = "****" if redacted_url.password
    raise ArgumentError, "Invalid scheme #{@url.scheme}. Must be https" unless @url.scheme == 'https'
    raise ArgumentError, "no user specified" unless @url.user
    raise ArgumentError, "no password specified" unless @url.password
    snmp = parse_snmp(@url.query)
    @transport = Puppet::Util::NetworkDevice::Transport_equallogic.new
    @transport.host = @url.host
    @transport.user = URI.decode(@url.user)
	@transport.password = URI.decode(asm_decrypt(@url.password))
    if (snmp.to_s.strip.length > 0)
      @transport.snmpcommunity = "#{snmp}"
    else
      @transport.snmpcommunity = "public"
    end
    libPath = "#{$provider_path}/lib"
    ENV['LD_LIBRARY_PATH'] = "#{libPath}"
    ENV['PS_GROUP_ADDR'] = "#{@transport.host}"
    ENV['PS_GROUP_USER'] = "#{@transport.user}"
    ENV['PS_GROUP_PASSWORD'] = "#{@transport.password}"
    response = checkConnection
    if (response.match("Error"))
      raise Puppet::Error, "Unable to set up a connection with the equallogic device because of the following exception: #{response}."
    end
  end

  def checkConnection
    response = `python #{$pythonFilePath} #{@transport.host} #{@transport.user} #{@transport.password} show`
    return response
  end

  def parse_snmp(query)
    if query
      params = CGI.parse(query)
      params['community_string'].first unless params['community_string'].empty?
    end
  end

  def facts
    @facts ||= Puppet::Util::NetworkDevice::Equallogic::Facts.new(@transport)
    facts = @facts.retrieve

    facts
  end
end
