require 'puppet/util/network_device/equallogic'
require 'puppet/util/network_device/transport_equallogic'
require 'json'

class Puppet::Util::NetworkDevice::Equallogic::Facts
  attr_reader :transport

  $provider_path = Pathname.new(__FILE__).parent
  $pythonFilePath = "#{$provider_path}/pythonEquallogic.py"
  $pythonSNMPFilePath = "#{$provider_path}/EquallogicSNMP.py"
  $volumeFilePath = "#{$provider_path}/volume"
  $poolFilePath = "#{$provider_path}/pool"
  $memberFilePath = "#{$provider_path}/member"
  def initialize(transport)
    @transport = transport
  end

  def retrieve
    @facts = {}
    @facts['General Settings'] = getgeneralsettinginfo
    @facts['Storage Pools'] = getstoragepoolinfo
    @facts['Group Members'] = getmembercountinfo
    volumeList = getvolumelist
    getvolumeproperties(volumeList)
    @facts['Collections'] = getcollectioninfo
    @facts['Model'] = "EqualLogic"
    @facts['Group Disk Space'] = getgroupdiskspace
    @facts['Free Group Space'] = getfreespace
    @facts['Volume_IQN_Information'] = get_volume_info_c
    @facts
  end
  
  def get_volume_info_c
    #Setting the access information in the environment
    ENV['PS_GROUP_ADDR'] = @transport.host
    ENV['PS_GROUP_USER'] = @transport.user
    ENV['PS_GROUP_PASSWORD'] = @transport.password
    ENV['LD_LIBRARY_PATH'] = "#{ENV['LD_LIBRARY_PATH']}:#{Pathname.new(__FILE__).parent}/lib"
    volumes_response = ASM::Util.run_command_simple("#{Pathname.new(__FILE__).parent}/volume show")
    vol_data_hash = {}
    volumes_response['stdout'].split("\n").each_with_index do |line,i|
      # Skip the first three lines of the response
      if (i <= 2 || line.length == 0 )
        next
      end

      line.split("\t").each do |vol_data|
        vol_name = vol_data[0].gsub(/\s+/, "")
        vol_data_hash["#{vol_name}"] ||= {}
        vol_data_hash["#{vol_name}"]['TargetIscsiName'] = vol_data[1].gsub(/\s+/, "")
        vol_data_hash["#{vol_name}"]['Size'] = vol_data[2].gsub(/\s+/, "")
        vol_data_hash["#{vol_name}"]['Snapshots'] = vol_data[3].gsub(/\s+/, "")
        vol_data_hash["#{vol_name}"]['Status'] = vol_data[4].gsub(/\s+/, "")
        vol_data_hash["#{vol_name}"]['Permission'] = vol_data[5].gsub(/\s+/, "")
        vol_data_hash["#{vol_name}"]['TP'] = vol_data[6].gsub(/\s+/, "")
        vol_data_hash["#{vol_name}"]['Template'] = vol_data[7].gsub(/\s+/, "")
      end
    end
    vol_data_hash
  end

  def getsnapshotreserveresponse
    response = `python "#{$pythonSNMPFilePath}" "#{@transport.host}" "#{@transport.snmpcommunity}" getsnapshotreserve`
    return response
  end

  def getsnapshotreserve
    snapReserve = "N/A"
    response = getsnapshotreserveresponse
    if (response.match("Failed") || response.match("No SNMP"))
      return snapReserve
    end
    snapReserve = response.to_s.strip
    snapReserve
  end

  def getreplicationreserveresponse
    response = `python "#{$pythonSNMPFilePath}" "#{@transport.host}" "#{@transport.snmpcommunity}" getreplicationreserve`
    return response
  end

  def getreplicationreserve
    replReserve = "N/A"
    response = getreplicationreserveresponse
    if (response.match("Failed") || response.match("No SNMP"))
      return replReserve
    end
    replReserve = response.to_s.strip
    replReserve
  end

  def getdelegatedresponse
    response = `python "#{$pythonSNMPFilePath}" "#{@transport.host}" "#{@transport.snmpcommunity}" getdelegated`
    return response
  end

  def getdelegated
    delegated = "N/A"
    response = getdelegatedresponse
    if (response.match("Failed") || response.match("No SNMP"))
      return delegated
    end
    delegated = response.to_s.strip
    delegated
  end

  def getfreespaceresponse
    response = `python "#{$pythonSNMPFilePath}" "#{@transport.host}" "#{@transport.snmpcommunity}" getfreespace`
    return response
  end

  def getfreespace
    freeSpace = "N/A"
    response = getfreespaceresponse
    if (response.match("Failed") || response.match("No SNMP"))
      return freeSpace
    end
    freeSpace = response.to_s.strip
    freeSpace
  end

  def getvolumereserveresponse
    response = `python "#{$pythonSNMPFilePath}" "#{@transport.host}" "#{@transport.snmpcommunity}" getvolumereserve`
    return response
  end

  def getvolumereserve
    volReserve = "N/A"
    response = getvolumereserveresponse
    if (response.match("Failed") || response.match("No SNMP"))
      return volReserve
    end
    volReserve = response.to_s.strip
    volReserve
  end

  def getgroupdiskspace
    @groupDiskSpaceMap = {}
    @groupDiskSpaceMap['Snapshot Reserve'] = getsnapshotreserve
    @groupDiskSpaceMap['Replication Reserve'] = getreplicationreserve
    @groupDiskSpaceMap['Delegated'] = getdelegated
    @groupDiskSpaceMap['Free'] = getfreespace
    @groupDiskSpaceMap['Volume Reserve'] = getvolumereserve
    JSON.pretty_generate(@groupDiskSpaceMap)
  end

  def getvolumeproperties(volumelist)
    @volumeMap = {}
    @volumeCountMap = {}
    @snapshotPropertiesMap = {}
    volumeCounter = 0
    onlineVolumeCounter = 0
    inuseVolumeCounter = 0
    iSCSIConnCounter = 0
    snapshotCounter = 0
    onlinesnapshotCounter = 0
    inusesnapshotCounter = 0
    # We are excluding recovery volumes, vss-control and pe-control-vol 
    # [XXX] Still do not know the purpose of pe-control-vol, as it not inside the EQL, UI or referenced on Google
    vol_exclude_re = /^(.*(-\d{4}-\d\d-\d\d-(\d\d:){2}\d\d\.\d+\.\d+)|vss-control|pe-control-vol)$/
    volumelist[:volumes].each_with_index do |volume,i| 
      next if volume['Name'].match(vol_exclude_re)
      vol = mapvolumeproperties(volume,volumelist[:volume_statuses][i])
      volumeCounter += 1
      onlineVolumeCounter += 1 if vol['Volume Status'] == 'online'
      inuseVolumeCounter += 1 if vol['Volume Use'] != '0'
      snapshotCounter += vol['Number of Snapshot'].to_i
      iSCSIConnCounter += vol['ISCSI Connections'].to_i
      onlinesnapshotCounter += vol['Number of Online Snapshots'].to_i
      inusesnapshotCounter += vol['Snapshots in Use'].to_i
      @volumeMap[volume['Name']] = JSON.pretty_generate(vol)
    end
    @facts['VolumesProperties'] = JSON.pretty_generate(@volumeMap)
    @volumeCountMap['Total Volumes'] = "#{volumeCounter}"
    @volumeCountMap['Online'] = "#{onlineVolumeCounter}"
    @volumeCountMap['ISCSI Connections'] = "#{iSCSIConnCounter}"
    @volumeCountMap['In Use'] = "#{inuseVolumeCounter}"
    @facts['VolumesInfo'] = JSON.pretty_generate(@volumeCountMap)
    @facts['Volumes'] = "#{volumeCounter}"
    @facts['Snapshots'] = "#{snapshotCounter}"
    @snapshotPropertiesMap['Online'] = "#{onlinesnapshotCounter}"
    @snapshotPropertiesMap['In Use'] = "#{inusesnapshotCounter}"
    @facts['SnapshotsInfo'] = JSON.pretty_generate(@snapshotPropertiesMap)
  end

  def mapvolumeproperties (volume, volume_status)
    volumePropertiesMap = {}
    snapshotPropertiesMap = {}
    volumePropertiesMap['Storage Pool'] = volume['StoragePoolIndex'] == "1" ? "default" : "Pool #{volume['StoragePoolIndex']}"
    volumePropertiesMap['Reported Size'] = (volume['Size'].to_f / 1024).to_s + "GB"
    volumePropertiesMap['Volume Reserve'] = (volume_status['StatusReservedSpaceAvail'].to_f / 1024).to_s + "GB"
    volumePropertiesMap['Snapshot Reserve'] = volume['ReservedPercentage'] + "%"
    volumePropertiesMap['Borrowed Space'] = volume['StatusReplReserveSpace'].to_i.to_s + "MB"
    volumePropertiesMap['Volume Status'] = volume['RowStatus'] == "1" ? "online" : "not online"
    volumePropertiesMap['Volume Use'] = volume_status['StatusConnections']
    volumePropertiesMap['Replication Partner'] = "" #to-verify
    volumePropertiesMap['SyncRep Status'] = "" #to-verify
    volumePropertiesMap['Number of Snapshot'] = volume_status['StatusNumSnapshots']
    volumePropertiesMap['Number of Online Snapshots'] = volume_status['StatusNumOnlineSnaps']
    volumePropertiesMap['ISCSI Connections'] = volume_status['StatusExternalConnections']
    volumePropertiesMap['Snapshots in Use'] = volume_status['StatusReplResvInUse']
    volumePropertiesMap['TargetIscsiName'] = volume['TargetIscsiName']
    volumePropertiesMap
  end

  def getcollectioninforesponse
    response = `python "#{$pythonFilePath}" #{@transport.host} #{@transport.user} #{@transport.password} volumecollectioninfoshow`
    return response
  end

  def getsnapcollectioninforesponse
    response = `python "#{$pythonFilePath}" #{@transport.host} #{@transport.user} #{@transport.password} snapcollectioninfoshow`
    return response
  end

  def getcollectioninfo
    @collectionMap = {}
    counter = 0
    volumeCollectionCounter = 0
    snapCollectionCounter = 0
    response = getcollectioninforesponse
    response.split("\n").each do |line|
      if (counter < 2)
        counter = counter + 1
        next
      end
      if (line.to_s.strip.length > 0 && line =~ /^\S+/ )
        volumeCollectionCounter = volumeCollectionCounter + 1
      end
    end
    counter = 0
    response = getsnapcollectioninforesponse
    response.split("\n").each do |line|
      if (counter < 2)
        counter = counter + 1
        next
      end
      if (line.to_s.strip.length > 0 && line =~ /^\S+/ )
        snapCollectionCounter = snapCollectionCounter + 1
      end
    end
    @collectionMap['Volume Collections'] = "#{volumeCollectionCounter}"
    @collectionMap['Custom Snapshot Collections'] = "#{snapCollectionCounter}"
    @collectionMap['Snapshot Collections'] = "#{@facts['Snapshots']}"
    JSON.pretty_generate(@collectionMap)
  end

  def getmemberinforesponse(member)
    response = `python "#{$pythonFilePath}" #{@transport.host} #{@transport.user} #{@transport.password} membershowfordiscovery "#{member}"`
    return response
  end

  def getmemberresponse
    response = `"#{$memberFilePath}" show`
    return response
  end

  def getmembercountinfo
    memberCounter = 0
    @memberMap = {}
    @memberStatusMap = {}
    @memberVersionMap = {}
    resp = getmemberresponse
    if (resp.match("% Error"))
      raise Puppet::Error, "Failed to fetch the equallogic pools: #{resp}"
    end
    memberModelList = Array.new
    resp.split("\n").each do |line|
      member = line.to_s.strip
      if ( member.size > 0 )
        memberCounter = memberCounter + 1
        response = getmemberinforesponse(member)
        if (response.match("% Error"))
          raise Puppet::Error, "Failed to fetch the equallogic member properties: #{response}"
        end
        response.split("\n").each do |line|
          res = line.split(":")
          name = res[0].to_s.strip
          value = res[1].to_s.strip
          if name == "Product Family"
            @memberMap[member] = value
          end
          if name == "Status"
            @memberStatusMap[member] = value
          end
          if name == "Version" 
            @memberVersionMap[member] = value
          end
        end

      end
    end
    @facts['Member Model'] = JSON.pretty_generate(@memberMap)
    @facts['Status'] = JSON.pretty_generate(@memberStatusMap)
    @facts['Version'] = JSON.pretty_generate(@memberVersionMap)
    memberCounter
  end

  def getvolumelist
    volumes = transport.get_volumes
    volume_statuses = transport.get_volume_statuses
    {:volumes => volumes, :volume_statuses => volume_statuses}
  end

  def getpoolresponse
    response = `"#{$poolFilePath}" show`
    return response
  end

  def getstoragepoolinfo
    poolMap = {}
    resp = getpoolresponse
    if (resp.match("% Error"))
      raise Puppet::Error, "Failed to fetch the equallogic pools: #{resp}"
    end
    counter = 0
    resp.split("\n").each do |line|
      if (counter < 3)
        counter = counter + 1
        next
      else
        res = line.split(' ')
        poolMap[res[0].to_s.strip] = {
        'Members' => res[2].to_s.strip,
        'Total' => res[4].to_s.strip}
      end
    end
    JSON.pretty_generate(poolMap)
  end

  def getgrpparamresponse
    response = `python "#{$pythonFilePath}" #{@transport.host} #{@transport.user} #{@transport.password} discoverygrpparamsshow`
    return response
  end

  def getgeneralsettinginfo
    @generalMap = {}
    @groupPropertiesMap = {}
    response = getgrpparamresponse
    if (response.match("% Error"))
      raise Puppet::Error, "Failed to fetch the equallogic group parameters: #{response}"
    end
    response.split("\n").each do |line|
      res = line.split(":")
      name = res[0].to_s.strip
      value = res[1].to_s.strip
      if name == "Name"
        @generalMap['Group Name'] = value
        @facts['Group Name'] = value
      elsif name == "Group-Ipaddress"
        @generalMap['IP Address'] = value
      elsif name == "Management-Ipaddress"
        @facts['Management IP'] = value
      elsif name == "Location"
        @generalMap['Location'] = value
      end
    end
    JSON.pretty_generate(@generalMap)
  end
end
