require 'net/https'
require 'puppet/util/network_device'

class Puppet::Util::NetworkDevice::Transport_equallogic
  attr_accessor :host, :user, :password, :snmpcommunity
  def initialize
  end

  def get_volumes
    vol_names = '1.3.6.1.4.1.12740.5.1.7.1.1.4'
    vol_props = '1.3.6.1.4.1.12740.5.1.7.1.1'
    volume_columns = [
      "Index",
      "RowStatus",
      "PsvId",
      "Name",
      "Site",
      "Description",
      "AccessType",
      "Size",
      "AdminStatus",
      "ReservedPercentage",
      "SnapWarningLevel",
      "SnapDeletionPolicy",
      "AutoloadBalancing",
      "TargetAlias",
      "TargetIscsiName",
      "NodeIndex",
      "HdrDigests",
      "DataDigests",
      "CloneSrcPsvId",
      "ReplService",
      "MultInitiator",
      "StoragePoolIndex",
      "StoragePoolSourceIndex",
      "TargetLocalMemberId",
      "TargetIndex",
      "ThinProvision",
      "MinThinReserve",
      "ThinWarnPercentage",
      "ThinMaxGrowPercentage",
      "ShrinkAutoAdjust",
      "ReplRollbackSeqNum",
      "ThinWarnMode",
      "Flags",
      "Template",
      "ThinClone",
      "ThinCloneParentMemberId",
      "ThinCloneParentVolIndex",
      "ThinCloneParentPsvId",
      "AdminAccountKey",
      "SCSIQErr"
    ]
    get_resources(host, snmpcommunity, vol_names, vol_props, volume_columns)
  end
  
  def get_volume_statuses
    vol_names = '1.3.6.1.4.1.12740.5.1.7.7.1.1'
    vol_props = '1.3.6.1.4.1.12740.5.1.7.7.1'
    volume_columns = [
      "StatusIndex",
      "StatusPsvId",
      "StatusReservedSpace",
      "StatusReservedSpaceAvail",
      "StatusActualMemberCount",
      "StatusNumSnapshots",
      "StatusCreationTime",
      "StatusAvailable",
      "StatusOperStatus",
      "StatusConnections",
      "StatusLostRaidBlocksAction",
      "StatusNumReplicas",
      "StatusReplReserveSpace",
      "StatusAllocatedSpace",
      "StatusReplResvInUse",
      "StatusReplTxData",
      "StatusNumOnlineSnaps",
      "StatusPagesSharedWithParent",
      "StatusExternalConnections"
    ]
    get_resources(host, snmpcommunity, vol_names, vol_props, volume_columns)
  end
  
  def get_resources(host, community, names_mib, volumes_mib, columns)
    require 'asm/util'
    Puppet.debug("Executing snmpwalk -v 2c -O vqe -c #{community} #{host} #{names_mib}")
    volume_count_response = ASM::Util.run_command_simple("snmpwalk -v 2c -O vqe -c #{community} #{host} #{names_mib}")
    if volume_count_response['exit_status'] == 0
      volume_count = volume_count_response['stdout'].split("\n").count
    else
      Puppet.debug(volume_count_response)
      raise Puppet::Error, volume_count_response['stderr']
    end
    Puppet.debug("Executing: snmpwalk -v 2c -O vqe -c #{community} #{host} #{volumes_mib}")
    volumes_response = ASM::Util.run_command_simple("snmpwalk -v 2c -O vqe -c #{community} #{host} #{volumes_mib}")
    if volumes_response['exit_status'] == 0
      volumes = volumes_response['stdout']
    else
      Puppet.debug(volumes_response)
      raise Puppet::Error, volumes_response['stderr']
    end
    p_index = 0
    p_list = []
    p_list[0] = []
    volumes.split("\n").each_with_index do |line,i|
      if (i != 0) && ((i % volume_count) == 0)
        p_index += 1
        p_list[p_index] = []
      end
      p_list[p_index].push(line)
    end
    v_list = []
    v_hash_list = []
    p_list.each_with_index do |property,i|
      property.each_with_index do |v_property,j|
        v_list[j] = [] if i == 0
        v_list[j].push(v_property.gsub /"/, '')
        v_hash_list[j] = {} if i == 0
        column_name = columns[v_list[j].count]
        v_hash_list[j][column_name] = v_property.gsub(/"/, '') unless column_name.nil?
      end
    end
    v_hash_list
  end
end
