#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'puppet/provider/equallogic'
require 'puppet/util/network_device/equallogic/device'


describe Puppet::Type.type(:equallogic_volume).provider(:equallogic_volume) do

	describe "State of Testing the functionality of Equallogic volume create and destroy." do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		before :each do
			setup_facter_value(device_conf['url'])
		end

		create_volume_yml =  YAML.load_file(my_fixture('create_volume.yml'))
		create_node1 = create_volume_yml['CreateVolume1']  
		let :create_volume do
			Puppet::Type.type(:equallogic_volume).new(
				:name          		=> create_node1['name'],
				:ensure        		=> create_node1['ensure'],
				:state				=> create_node1['state'],
				:multihostaccess 	=> create_node1['multihostaccess'],
				:initsize			=> create_node1['size'],
				:thinprovision		=> create_node1['thinprovision'],
				:snapreserve		=> create_node1['snapresv'],
				:thinminreserve		=> create_node1['thinminreserve'],
				:thingrowthwarn		=> create_node1['thingrowthwarn'],
				:thingrowthmax		=> create_node1['thingrowthmax'],
				:thinwarnsoftthres	=> create_node1['thinwarnsoftthres'],
				:thinwarnhardthres	=> create_node1['thinwarnhardthres'],
				:poolname			=> create_node1['poolname'],
			)
		end

		create_node2 = create_volume_yml['CreateVolume2']
		let :create_volume_oneK do
			Puppet::Type.type(:equallogic_volume).new(
				:name          		=> create_node2['name'],
				:ensure        		=> create_node2['ensure'],
				:state				=> create_node2['state'],
				:multihostaccess 	=> create_node2['multihostaccess'],
				:initsize			=> create_node2['size'],
				:thinprovision		=> create_node2['thinprovision'],
				:snapreserve		=> create_node2['snapresv'],
				:thinminreserve		=> create_node2['thinminreserve'],
				:thingrowthwarn		=> create_node2['thingrowthwarn'],
				:thingrowthmax		=> create_node2['thingrowthmax'],
				:thinwarnsoftthres	=> create_node2['thinwarnsoftthres'],
				:thinwarnhardthres	=> create_node2['thinwarnhardthres'],
				:poolname			=> create_node2['poolname'],
			)
		end

		# Invalid name
		create_node3 = create_volume_yml['CreateVolume3']
		let :create_volume_InvalidName do
		Puppet::Type.type(:equallogic_volume).new(
			:name          		=> create_node3['name'],
			:ensure        		=> create_node3['ensure'],
			:state				=> create_node3['state'],
			:multihostaccess 	=> create_node3['multihostaccess'],
			:initsize			=> create_node3['size'],
			:thinprovision		=> create_node3['thinprovision'],
			:snapreserve		=> create_node3['snapresv'],
			:thinminreserve		=> create_node3['thinminreserve'],
			:thingrowthwarn		=> create_node3['thingrowthwarn'],
			:thingrowthmax		=> create_node3['thingrowthmax'],
			:thinwarnsoftthres	=> create_node3['thinwarnsoftthres'],
			:thinwarnhardthres	=> create_node3['thinwarnhardthres'],
			:poolname			=> create_node3['poolname'],
		)
		end

		create_node4 = create_volume_yml['CreateVolume4']
		let :create_volume_Gmax do
			Puppet::Type.type(:equallogic_volume).new(
				:name          		=> create_node4['name'],
				:ensure        		=> create_node4['ensure'],
				:state				=> create_node4['state'],
				:multihostaccess 	=> create_node4['multihostaccess'],
				:initsize			=> create_node4['size'],
				:thinprovision		=> create_node4['thinprovision'],
				:snapreserve		=> create_node4['snapresv'],
				:thinminreserve		=> create_node4['thinminreserve'],
				:thingrowthwarn		=> create_node4['thingrowthwarn'],
				:thingrowthmax		=> create_node4['thingrowthmax'],
				:thinwarnsoftthres	=> create_node4['thinwarnsoftthres'],
				:thinwarnhardthres	=> create_node4['thinwarnhardthres'],
				:poolname			=> create_node4['poolname'],
			)
		end

		destroy_volume_yml =  YAML.load_file(my_fixture('destoy_volume.yml'))
		destroy_node1 = destroy_volume_yml['DestroyVolume1']

		let :destroy_volume do
			Puppet::Type.type(:equallogic_volume).new(
				:name          		=> destroy_node1['name'],
				:ensure        		=> destroy_node1['ensure'],
			)
		end

		destroy_node2 = destroy_volume_yml['DestroyVolume2']
		let :destroy_volume_NonExists do
			Puppet::Type.type(:equallogic_volume).new(
				:name          		=> destroy_node2['name'],
				:ensure        		=> destroy_node2['ensure'],
			)
		end

		destroy_node3 = destroy_volume_yml['DestroyVolume3']
		let :destroy_volume_Invalid do
			Puppet::Type.type(:equallogic_volume).new(
				:name          		=> destroy_node3['name'],
				:ensure        		=> destroy_node3['ensure'],
			)
		end

		destroy_node4 = destroy_volume_yml['DestroyVolume4']
		let :destroy_volume_Null do
			Puppet::Type.type(:equallogic_volume).new(
				:name          		=> destroy_node4['name'],
				:ensure        		=> destroy_node4['ensure'],
			)
		end

		let :provider do
			described_class.new( )
		end

		describe "when creating a new volume - with unique name" do
			it "should be able to create a volume" do
				create_volume.provider.create
			end
			it "Validation : Create volume opeartion is successful - volume is exists in equallogic." do
				create_verify_response = verify_volume(create_node1['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when creating a new volume - with name already exists" do
			it "should not be allowed to create a volume with duplicate name" do
				begin
					create_volume.provider.create
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Failed to Create Volume/)}.not_to raise_error
				end
			end
			it "Validation : Create volume opeartion - when volume already exists - volume is exists in equallogic." do
				create_verify_response = verify_volume(create_node1['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when creating a volume with 1k size - it should fail" do
			it "should not be allowed  to create a volume with 1k size" do
				begin
					create_volume_oneK.provider.create
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Parameter initsize failed/)}.not_to raise_error
				end
			end
			it "Validation : Create volume opeartion 1k size not created - volume should not exists in equallogic." do
				create_verify_response = verify_volume(create_node2['name'])
				expect{expect(create_verify_response).not_to be_true}.to raise_error 
			end
		end

		describe "when creating a new volume - with Invalid Name" do
			it "should not be allowed to create a volume with Invalid name" do
				begin
					create_volume_InvalidName.provider.create
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Parameter name failed/)}.not_to raise_error
				end
			end
			it "Validation : Create volume opeartion - Invalid Name - volume should not exists in equallogic." do
				create_verify_response = verify_volume(create_node3['name'])
				expect{expect(create_verify_response).not_to be_true}.to raise_error 
			end
		end

		describe "when creating a new volume - with morethan Max characters exists" do
			it "should not be allowed to create a volume with morethan Max characters name" do
				begin
					create_volume_Gmax.provider.create
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Parameter name failed/)}.not_to raise_error
				end
			end
			it "Validation : Create volume opeartion - morethan Max characters - volume should not exists in equallogic." do
				create_verify_response = verify_volume(create_node4['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when destroying a volume - which is exists" do
			it "should be able to destroy a volume with valid values" do
				destroy_volume.provider.destroy
			end
			it "Validation : Destroy volume opeartion successful - volume should not exists in equallogic." do
				destroy_verify_response = verify_volume(destroy_node1['name'])
				expect{expect(destroy_verify_response).not_to be_true}.to raise_error 
			end
		end

		describe "when destroying a volume which non existant" do
			it "destroy should not be successful if volume does not exists" do
				begin
					destroy_volume_NonExists.provider.destroy
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Failed to/)}.not_to raise_error
				end
			end
			it "Validation : Destroy volume opeartion not executed - volume should not exists in equallogic." do
				destroy_verify_response = verify_volume(destroy_node2['name'])
				expect{expect(destroy_verify_response).not_to be_true}.to raise_error 
			end
		end

		describe "when destroying a volume - volume name value is invalid" do
			it "should not be able to destroy a volume - name value is invalid" do
				begin
					destroy_volume_Invalid.provider.destroy
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Parameter name/)}.not_to raise_error
				end
			end
			it "Validation : Destroy volume opeartion not executed - volume should not exists in equallogic." do
				destroy_verify_response = verify_volume(destroy_node3['name'])
				expect{expect(destroy_verify_response).not_to be_true}.to raise_error 
			end
		end

		describe "when destroying a volume - volume name value is null" do
			it "destroy should not be successful if volume name value is null" do
				begin
					destroy_volume_Null.provider.destroy
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Parameter name/)}.not_to raise_error
				end
			end
			it "Validation : Destroy volume opeartion not executed - volume should not exists in equallogic." do
				destroy_verify_response = verify_volume(destroy_node4['name'])
				expect{expect(destroy_verify_response).not_to be_true}.to raise_error 
			end
		end
	end
	
end
