#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'puppet/provider/equallogic'
require 'puppet/util/network_device/equallogic/device'


describe Puppet::Type.type(:equallogic_volume).provider(:equallogic_volume) do

	describe "Volume state change - settingup - required volume creation started" do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		volume_state =  YAML.load_file(my_fixture('volume_state.yml'))
		state_node1 = volume_state['VolumeState1']
		before :all do
			setup_method(device_conf['url'],state_node1['name'])
		end
		describe "required volume creation start " do
			it "should be able to create required volume" do
					##Setup creation started
			end
		end
	end

	describe "Equallogic volume - Volume offline and online state change validation" do

		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		before :each do
			setup_facter_value(device_conf['url'])
		end				  

		volume_state =  YAML.load_file(my_fixture('volume_state.yml'))
		state_node1 = volume_state['VolumeState1']
		let :offline_to_online do
			Puppet::Type.type(:equallogic_volume).new(
			:name          		=> state_node1['name'],
			:ensure        		=> state_node1['ensure'],
			:state				=> state_node1['state'],
			)
		end

		state_node2 = volume_state['VolumeState2']
		let :online_to_offline do
			Puppet::Type.type(:equallogic_volume).new(
			:name          		=> state_node2['name'],
			:ensure        		=> state_node2['ensure'],
			:state				=> state_node2['state'],
			)
		end

		let :provider do
			described_class.new( )
		end

		describe "when changing state of volume from offline to online" do
			it "change state of volume from offline to online" do
				online_to_offline.provider.state='offline'
			end
			it "Validation : Change volume state from online to offline opeartion successful - volume exists in equallogic." do
				create_verify_response = verify_volume_property('state', state_node2['state'], state_node2['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when changing state of volume from offline to online" do
			it "change state of volume from offline to online" do
				offline_to_online.provider.state='online'
			end
			it "Validation : Change volume state from offline to online opeartion successful - volume exists in equallogic." do
				create_verify_response = verify_volume_property('state', state_node1['state'], state_node1['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error
			end
		end

	end

	describe "Volume state change - cleaningup - volume created for testing is destoryed" do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		volume_state =  YAML.load_file(my_fixture('volume_state.yml'))
		state_node1 = volume_state['VolumeState1']
		before :all do
			cleanup_method(device_conf['url'],state_node1['name'])
		end
		describe "volume created for testing destory operation start " do
			it "should be able to destroy created volume" do
					##Cleanup volume started
			end
		end
	end

end
