#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'


describe Puppet::Type.type(:equallogic_volume_access_chap).provider(:equallogic_volume_access_chap) do
	
	describe "settingup - required volume creation started" do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		create_access_yml =  YAML.load_file(my_fixture('create_volume_access_chap.yml'))
		create_node1 = create_access_yml['CreateAccessChap1']
		before :all do
			setup_method(device_conf['url'],create_node1['name'])
		end
		describe "required volume creation start " do
			it "should be able to create required volume" do
					##Setup creation started
			end
		end
	end

	describe "State of Testing the functionality of Equallogic volume access chap." do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		before :each do
			setup_facter_value(device_conf['url'])
		end

		create_access_yml =  YAML.load_file(my_fixture('create_volume_access_chap.yml'))
		create_node1 = create_access_yml['CreateAccessChap1']
		let :create_volume_access_chap do
			Puppet::Type.type(:equallogic_volume_access_chap).new(
			:name          		=> create_node1['name'],
			:ensure        		=> create_node1['ensure'],
			:chap_user_name 	=> create_node1['chap_user_name'],
			)
		end

		create_node2 = create_access_yml['CreateAccessChap2']
		let :create_volume_access_chap_VolNonExists do
			Puppet::Type.type(:equallogic_volume_access_chap).new(
			:name          		=> create_node2['name'],
			:ensure        		=> create_node2['ensure'],
			:chap_user_name 	=> create_node2['chap_user_name'],
			)
		end

		destroy_access_yml =  YAML.load_file(my_fixture('destroy_volume_access_chap.yml'))
		destroy_node1 = destroy_access_yml['DestroyAccessChap1']
		let :destroy_volume_access_chap do
			Puppet::Type.type(:equallogic_volume_access_chap).new(
			:name          		=> destroy_node1['name'],
			:ensure        		=> destroy_node1['ensure'],
			:chap_user_name 	=> destroy_node1['chap_user_name'],
			)
		end

		destroy_node2 = destroy_access_yml['DestroyAccessChap2']
		let :destroy_volume_access_chap_VolNonExists do
			Puppet::Type.type(:equallogic_volume_access_chap).new(
			:name          		=> destroy_node2['name'],
			:ensure        		=> destroy_node2['ensure'],
			:chap_user_name 	=> destroy_node2['chap_user_name'],
			)
		end

		let :provider do
			described_class.new( )
		end

		describe "when creating a new chap access for volume - with unique name" do
			it "should be able to create new chap access with a volume" do
				create_volume_access_chap.provider.create
			end
			it "Validation : Create new chap access opeartion successful - and volume exists in equallogic." do
				create_verify_response = verify_volaccess_chap(create_node1['chap_user_name'], create_node1['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when creating a new chap access for non existing volume - with unique name" do
			it "should not be allowed to create a chap access with volume, which is not exists" do
				begin
					create_volume_access_chap_VolNonExists.provider.create
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Unable to create the chap access to the volume/)}.not_to raise_error
				end
			end
			it "Validation : Create new chap access opeartion not successful - and volume is not exists in equallogic." do
				create_verify_response = verify_volaccess_chap(create_node2['chap_user_name'], create_node2['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error
			end
		end

		describe "when destroying a chap access for volume - which is already exists" do
			it "should be able to destroy a chap access with volume with valid values" do
				destroy_volume_access_chap.provider.destroy
			end
			it "Validation : Destroy chap access opeartion successful - and volume is exists in equallogic." do
				destroy_verify_response = verify_volaccess_chap(destroy_node1['chap_user_name'], destroy_node1['name'])
				expect{expect(destroy_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when destroying a chap access for volume - volume which is non existant" do
			it "destroy of chap access with volume, should not be successful if volume does not exists" do
				begin
					destroy_volume_access_chap_VolNonExists.provider.destroy
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Unable to retrieve the chap access for the volume/)}.not_to raise_error
				end
			end
			it "Validation : Destroy chap access opeartion is not successful - and volume is not exists in equallogic." do
				destroy_verify_response = verify_volaccess_chap(destroy_node2['chap_user_name'], destroy_node2['name'])
				expect{expect(destroy_verify_response).to be_true}.not_to raise_error 
			end
		end
	end

	describe "cleaningup - volume created for testing is destoryed" do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		create_access_yml =  YAML.load_file(my_fixture('create_volume_access_chap.yml'))
		create_node1 = create_access_yml['CreateAccessChap1']
		before :all do
			cleanup_method(device_conf['url'],create_node1['name'])
		end
		describe "volume created for testing destory operation start " do
			it "should be able to destroy created volume" do
					##Cleanup volume started
			end
		end
	end
	
end
