#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'


describe Puppet::Type.type(:equallogic_volume_access_iqn).provider(:equallogic_volume_access_iqn) do

	describe "settingup - required volume creation started" do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		create_access_yml =  YAML.load_file(my_fixture('create_volume_access_iqn.yml'))
		create_node1 = create_access_yml['CreateAccessIQN1'] 
		before :all do
			setup_method(device_conf['url'],create_node1['name'])
		end
		describe "required volume creation start " do
			it "should be able to create required volume" do
					##Setup creation started
			end
		end
	end
	
	describe "State of Testing the functionality of Equallogic volume access iqn." do
	
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		before :each do
			setup_facter_value(device_conf['url'])
		end

		create_access_yml =  YAML.load_file(my_fixture('create_volume_access_iqn.yml'))
		create_node1 = create_access_yml['CreateAccessIQN1'] 
		let :create_volume_access_iqn do
			Puppet::Type.type(:equallogic_volume_access_iqn).new(
			:name          		=> create_node1['name'],
			:ensure        		=> create_node1['ensure'],
			:iqnorip       		=> create_node1['iqnorip'],
			)
		end

		create_node2 = create_access_yml['CreateAccessIQN2']
		let :create_volume_access_iqn_VolNonExists do
			Puppet::Type.type(:equallogic_volume_access_iqn).new(
			:name          		=> create_node2['name'],
			:ensure        		=> create_node2['ensure'],
			:iqnorip       		=> create_node2['iqnorip'],
			)
		end

		destroy_access_yml =  YAML.load_file(my_fixture('destroy_volume_access_iqn.yml'))
		destroy_node1 = destroy_access_yml['DestroyAccessIQN1']
		let :destroy_volume_access_iqn do
			Puppet::Type.type(:equallogic_volume_access_iqn).new(
			:name          		=> destroy_node1['name'],
			:ensure        		=> destroy_node1['ensure'],
			:iqnorip       		=> destroy_node1['iqnorip'],
			)
		end

		destroy_node2 = destroy_access_yml['DestroyAccessIQN2']
		let :destroy_volume_access_iqn_VolNonExists do
			Puppet::Type.type(:equallogic_volume_access_iqn).new(
			:name          		=> destroy_node2['name'],
			:ensure        		=> destroy_node2['ensure'],
			:iqnorip       		=> destroy_node2['iqnorip'],
			)
		end

		let :provider do
			described_class.new( )
		end

		describe "when creating a new iqn access for volume - with unique name" do
			it "should be able to create a iqn access with volume" do
				create_volume_access_iqn.provider.create
			end
			it "Validation : Create new iqn access opeartion successful - and volume exists in equallogic." do
				create_verify_response = verify_volaccess_iqn(create_node1['iqnorip'], create_node1['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when creating a new iqn access for volume - with a volume which non exists" do
			it "should not be allowed to create a iqn access with volume, where volume non exists" do
				begin
					create_volume_access_iqn_VolNonExists.provider.create
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Unable to create the iqn access to/)}.not_to raise_error
				end
			end
			it "Validation : Create new iqn access opeartion not successful - and volume is not exists in equallogic." do
				create_verify_response = verify_volaccess_iqn(create_node2['iqnorip'], create_node2['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error
			end
		end

		describe "when destroying a iqn access with volume - which is already exists" do
			it "should be able to destroy a iqn access with volume with valid values" do
				destroy_volume_access_iqn.provider.destroy
			end
			it "Validation : Destroy iqn access opeartion successful - and volume is exists in equallogic." do
				destroy_verify_response = verify_volaccess_iqn(destroy_node1['iqnorip'], destroy_node1['name'])
				expect{expect(destroy_verify_response).not_to be_true}.to raise_error
			end
		end

		describe "when destroying a iqn access with volume which non existant" do
			it "destroy of iqn access with volume, should not be successful if volume does not exists" do
				begin
					destroy_volume_access_iqn_VolNonExists.provider.destroy
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Unable to retrieve the iqn access for/)}.not_to raise_error
				end
			end
			it "Validation : Destroy iqn access opeartion is not successful - and volume is not exists in equallogic." do
				destroy_verify_response = verify_volaccess_iqn(destroy_node2['iqnorip'], destroy_node2['name'])
				expect{expect(destroy_verify_response).not_to be_true}.to raise_error
			end
		end
	end
	
	describe "cleaningup - volume created for testing is destoryed" do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		create_access_yml =  YAML.load_file(my_fixture('create_volume_access_iqn.yml'))
		create_node1 = create_access_yml['CreateAccessIQN1']
		before :all do
			cleanup_method(device_conf['url'],create_node1['name'])
		end
		describe "volume created for testing destory operation start " do
			it "should be able to destroy created volume" do
					##Cleanup volume started
			end
		end
	end
	
end


