#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'


describe Puppet::Type.type(:equallogic_volume_chap_user).provider(:equallogic_volume_chap_user) do

	describe "State of Testing the functionality of Equallogic volume create chapuser." do
	
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		before :each do
			setup_facter_value(device_conf['url'])
		end

		create_chapuser_yml =  YAML.load_file(my_fixture('create_chap_user.yml'))
		create_node1 = create_chapuser_yml['CreateChapUser1'] 
		let :create_chap_user do
			Puppet::Type.type(:equallogic_volume_chap_user).new(
			:name          		=> create_node1['name'],
			:ensure        		=> create_node1['ensure'],
			:chap_password   	=> create_node1['chap_password'],
			#:state				=> create_node1['state'],
		)
		end

		create_node2 = create_chapuser_yml['CreateChapUser2'] 
		let :create_chap_user_Invalid do
			Puppet::Type.type(:equallogic_volume_chap_user).new(
			:name          		=> create_node2['name'],
			:ensure        		=> create_node2['ensure'],
			:chap_password   	=> create_node2['chap_password'],
			#:state				=> create_node2['state'],
			)
		end

		destroy_chapuser_yml =  YAML.load_file(my_fixture('destroy_chap_user.yml'))
		destroy_node1 = destroy_chapuser_yml['DestroyChapUser1']
		let :destroy_chap_user do
			Puppet::Type.type(:equallogic_volume_chap_user).new(
			:name          		=> destroy_node1['name'],
			:ensure        		=> destroy_node1['ensure'],
			:chap_password   	=> destroy_node1['chap_password'],
			#:state				=> destroy_node1['state'],
			)
		end

		destroy_node2 = destroy_chapuser_yml['DestroyChapUser2']
		let :destroy_chap_user_Invalid do
			Puppet::Type.type(:equallogic_volume_chap_user).new(
			:name          		=> destroy_node2['name'],
			:ensure        		=> destroy_node2['ensure'],
			:chap_password   	=> destroy_node2['chap_password'],
			#:state				=> destroy_node2['state'],
			)
		end

		let :provider do
			described_class.new( )
		end

		describe "when asking exists?" do
			it "should return true if new chap user is present" do
				create_chap_user.provider.set(:ensure => :present)
				create_chap_user.provider.should_not be_exists
			end
		end

		describe "when creating a new chap user - with new user name" do
			it "should be able to create a new chap user" do
				create_chap_user.provider.create
			end
			it "Validation : Creating new chap user is successful - and chapuser exists in equallogic." do
				create_verify_response = verify_chapuser(create_node1['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when creating a new chap user - with invalid name " do
			it "should not be allowed to create a new chap user with invalid name" do
				begin
					create_chap_user_Invalid.provider.create
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Parameter name failed/)}.not_to raise_error
				end
			end
			it "Validation : Creating new chap user is not successful - and chapuser not exists in equallogic." do
				create_verify_response = verify_chapuser(create_node2['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when destroying a chap user - with new user name" do
			it "should be able to destroy a chap user" do
				destroy_chap_user.provider.destroy
			end
			it "Validation : Desstroy chap user is successful - and existing chapuser removed in equallogic." do
				destroy_verify_response = verify_chapuser(destroy_node1['name'])
				expect{expect(destroy_verify_response).not_to be_true}.to raise_error 
			end
		end

		describe "when destroying a chap user - with invalid/non-existing name " do
			it "should not be allowed to delete a chap user with invalid/non-existing name" do
				begin
					destroy_chap_user_Invalid.provider.destroy
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Parameter name failed/)}.not_to raise_error
				end
			end
			it "Validation : Desstroy chap user is not successful - and chapuser not exists in equallogic." do
				destroy_verify_response = verify_chapuser(destroy_node2['name'])
				expect{expect(destroy_verify_response).not_to be_true}.to raise_error 
			end
		end
			  
	end
	
end
