#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'


describe Puppet::Type.type(:equallogic_volume_get_iqn).provider(:equallogic_volume_get_iqn) do

	describe "settingup - required volume creation started" do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		get_volume_iqn_yml =  YAML.load_file(my_fixture('get_volume_iqn.yml'))
		getiqn_node1 = get_volume_iqn_yml['GetVolumeIQN1']
		before :all do
			setup_method(device_conf['url'],getiqn_node1['name'])
		end
		describe "required volume creation start " do
			it "should be able to create required volume" do
					##Setup creation started
			end
		end
	end
	
	describe "State of Testing the functionality of Equallogic volume get iqn." do

		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		before :each do
			setup_facter_value(device_conf['url'])
		end

		get_volume_iqn_yml =  YAML.load_file(my_fixture('get_volume_iqn.yml'))
		getiqn_node1 = get_volume_iqn_yml['GetVolumeIQN1'] 
		let :get_iqn_volume_present do
			Puppet::Type.type(:equallogic_volume_get_iqn).new(
			:name          		=> getiqn_node1['name'],
			:ensure        		=> getiqn_node1['ensure'],
			)
		end

		getiqn_node2 = get_volume_iqn_yml['GetVolumeIQN2']
		let :get_iqn_volume_absent do
			Puppet::Type.type(:equallogic_volume_get_iqn).new(
			:name          		=> getiqn_node2['name'],
			:ensure        		=> getiqn_node2['ensure'],
			)
		end

		let :provider do
			described_class.new( )
		end

		describe "when expecting iqn value of volume" do
			it "get iqn should be successful - if volume exists in equallogic." do
				get_iqn_volume_present.provider.create
			end
			it "Validation : Get iqn from volume opeartion successful - and volume exists in equallogic." do
				create_verify_response = verify_get_iqn(getiqn_node1['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end

		describe "when try to get iqn of volume, which non existant" do
			it "get iqn should not be successful - if volume does not exists" do
				begin
					get_iqn_volume_absent.provider.create
				rescue Exception => ex
					puts "Puppet Error expected : #{ex.message}"
					eresponse = ex.message
					expect{expect(eresponse).to match(/Failed to show IQN Info to the Volume/)}.not_to raise_error
				end
			end
			it "Validation : Get iqn from volume failed - and volume not exists in equallogic." do
				create_verify_response = verify_get_iqn(getiqn_node2['name'])
				expect{expect(create_verify_response).to be_true}.not_to raise_error 
			end
		end
		
	end
	
	describe "cleaningup - volume created for testing is destoryed" do
		device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
		get_volume_iqn_yml =  YAML.load_file(my_fixture('get_volume_iqn.yml'))
		getiqn_node1 = get_volume_iqn_yml['GetVolumeIQN1']
		before :all do
			cleanup_method(device_conf['url'],getiqn_node1['name'])
		end
		describe "volume created for testing destory operation start " do
			it "should be able to destroy created volume" do
					##Cleanup volume started
			end
		end
	end
	
end


