#! /usr/bin/env ruby

require 'spec_helper'
require 'puppet/util/network_device/equallogic/device'
require 'puppet/util/network_device/equallogic/facts'
require 'puppet/util/network_device/transport_equallogic'
require 'yaml'
require 'uri'

describe Puppet::Util::NetworkDevice::Equallogic::Device do

	device_conf =  YAML.load_file(my_deviceurl('equallogic','device_conf.yml'))
	describe "when connecting to a new device" do

		it "should reject a single hostname" do
			begin
				response = described_class.new('pfiler.example.com')
			rescue Exception => ex
				puts "Puppet Error expected : #{ex.message}"
				eresponse = ex.message
				expect{expect(eresponse).to match(/Invalid scheme/)}.not_to raise_error
			end
			
		end
		it "should reject a missing username" do
			begin
				response = described_class.new('https://pfiler.example.com')
			rescue Exception => ex
				puts "Puppet Error expected : #{ex.message}"
				eresponse = ex.message
				expect{expect(eresponse).to match(/no user specified/)}.not_to raise_error
			end
		end
		it "should reject a missing password" do
			begin
				response = described_class.new('https://root@pfiler.example.com')
			rescue Exception => ex
				puts "Puppet Error expected : #{ex.message}"
				eresponse = ex.message
				expect{expect(eresponse).to match(/no password specified/)}.not_to raise_error
			end
		end
		it "should not accept plain http connections" do
			begin
				response = described_class.new('http://root@pfiler.example.com')
			rescue Exception => ex
				puts "Puppet Error expected : #{ex.message}"
				eresponse = ex.message
				expect{expect(eresponse).to match(/Invalid scheme http/)}.not_to raise_error
			end
		end
		it "should not connect to the device equllogic - device not accessible" do
			begin
				response = described_class.new('https://useradmin:userpassword@192.168.1.100')
			rescue Exception => ex
				puts "Puppet Error expected : #{ex.message}"
				eresponse = ex.message
				expect{expect(eresponse).to match(/Unable to set up a connection with the equallogic device/)}.not_to raise_error
			end
		end

		context "#urlparse" do
			it "should parse url successfull" do
				Puppet::Util::Log.level = :debug
				Puppet::Util::Log.newdestination(:console)
				puts "parse url started"
				deviceobj =  described_class.new(device_conf['url'])
				transobj = deviceobj.transport
				puts transobj.host
				puts transobj.user
				puts transobj.password
				puts "parse url ended"
			end
		end

		context "#facts" do
			it "should connect to the device equllogic and retrive facts" do
				Puppet::Util::Log.level = :debug
				Puppet::Util::Log.newdestination(:console)
				puts "start facts"
				deviceobj =  described_class.new(device_conf['url'])
				transportobj = deviceobj.transport
				puts transportobj.host
				puts transportobj.user
				puts transportobj.password
				factsobj = Puppet::Util::NetworkDevice::Equallogic::Facts.new(transportobj)
				puts factsobj.retrieve
				puts "facts ended"
			end
		end

	end
end
