require 'uri'
require 'net/https'
require 'yaml'
require 'pathname'
require 'puppet/util/network_device/transport_equallogic'

$LOAD_PATH.unshift File.expand_path(File.dirname(__FILE__))
require "deviceconf"

module PuppetSpec::Validation
  $validationFile_path = Pathname.new(__FILE__).parent
  $pythonFilePath = "#{$validationFile_path}/../../../lib/puppet/util/network_device/equallogic/pythonEquallogic.py"
  $deviceConfPath = "#{$validationFile_path}/../../fixtures/device/device_conf.yml"
  $device_conf =  YAML.load_file($deviceConfPath)

  def url_parse(url)
    res = url.split("//")
    temp = res[1]
    output = temp.split(":")
    username = output[0]
    text = output[1]
    pos = text.rindex('@')
    length = text.length()
    password = text[0,pos]
    host = text[pos+1, length]  
    transportobj = Puppet::Util::NetworkDevice::Transport_equallogic.new
    transportobj.host = host
    transportobj.user = URI.decode(username)
    transportobj.password = URI.decode(password)
	return transportobj
  end
  def verify_volume(volname)
	$transport = url_parse($device_conf['url'])
    response = `python "#{$pythonFilePath}" #{$transport.host} #{$transport.user} #{$transport.password} volumeshow #{volname}`
	if (response.match("Error"))
      return "Failed to retrieve volume information.#{response}"
	  #return false
	else
	  return true
    end
    #return response
  end
  def verify_volume_property(volproperty, propertyvalue, volname)
	$transport = url_parse($device_conf['url'])
    response = `python "#{$pythonFilePath}" #{$transport.host} #{$transport.user} #{$transport.password} volumeshow #{volname}`
	if (response.match("Error"))
      return "Failed to retrieve volume information.#{response}"
	else
		volume_info = []
		volume_size = ""
		volume_name = ""
		volume_access = ""
		volume_template = ""
		volume_snap = ""
		volume_conn = ""
		volume_permission = ""
		volume_status = ""
		volume_thinprovision = ""
		volume_snapreserve = ""
		volume_thinminreserve = ""
		volume_thingrowthwarn = ""
		volume_thingrowthmax = ""
		#volume_thinwarnsoftthres = ""
		#volume_thinwarnhardthres = ""
		response.split("\n").each do |line|
		  res = line.split(":")
		  name = res[0].to_s.strip
		  value = res[1].to_s.strip
		  volume_status = get_status(name,value,volume_status)
		  volume_size = get_size(name,value,volume_size)
		  volume_name = get_name(name,value,volume_name)
		  volume_access = get_access(name,value,volume_access)
		  volume_template = get_template(name,value,volume_template)
		  volume_snap = get_snap(name,value,volume_snap)
		  volume_conn = get_conn(name,value,volume_conn)
		  volume_permission = get_permission(name,value,volume_permission)
		  volume_thinprovision = get_thinprovision(name,value,volume_thinprovision)
		  volume_snapreserve = get_snapreserve(name,value,volume_snapreserve)
		  volume_thinminreserve	= get_thinminreserve(name,value,volume_thinminreserve)
		  volume_thingrowthwarn	= get_thingrowthwarn(name,value,volume_thingrowthwarn)
		  volume_thingrowthmax	= get_thingrowthmax(name,value,volume_thingrowthmax)
		  #volume_thinwarnsoftthres	= get_thinwarnsoftthres(name,value,volume_thinwarnsoftthres)
		  #volume_thinwarnhardthres	= get_thinwarnhardthres(name,value,volume_thinwarnhardthres)
		end
		
		if ( volproperty == 'state')
			if (volume_status.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'initsize')
			if (volume_size.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'volname')
			if (volume_name.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'multihostaccess')
			if (volume_access.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'template')
			if (volume_template.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'snap')
			if (volume_snap.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'connections')
			if (volume_conn.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'permissions')
			if (volume_permission.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'thinprovision')
			if (volume_thinprovision.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'snapreserve')
			if (volume_snapreserve.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'thinminreserve')
			if (volume_thinminreserve.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'thingrowthwarn')
			if (volume_thingrowthwarn.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'thingrowthmax')
			if (volume_thingrowthmax.match(propertyvalue))
				return true
			end
		else
			return false
		end
	end
			
  end
  def verify_volaccess_chap(chapuserlist, volname)
    chap_user_list = chapuserlist
	chap_users = chap_user_list.split(",")
	chap_users_list = check_chap_users(chap_users, volname)
	if (chap_users_list.size == 0)
	   #return true
	   return false
	else
	   #return false
	   return true
	end
  end
  def check_chap_users(chap_user_list, volname)
	$transport = url_parse($device_conf['url'])
    response = ` python "#{$pythonFilePath}" #{$transport.host} #{$transport.user} #{$transport.password} volumeaccessshow #{volname}`
    if (response.match("% Error"))
      return "Unable to retrieve the chap access for the volume #{volname} because of the following exception: #{response}."
    end
    counter = 0
    response.split("\n").each do |line|
      if (counter < 3)
        counter = counter + 1
        next
      end
      if (line.to_s.strip.length == 0)
        next
      end
      res = line.to_s.strip.split(" ")
      tempid = res[0]
      iq = res[1]
      temp_user_name = res[3]
      tempip = res[2]
      if (line.to_s.strip.length == 0)
        next
      end

      if (tempid.to_s.strip =~ /^\d+$/ && !(line.strip.match("iqn")))
        iqnum = "#{iq}"
        $id = "#{tempid}"
        ip = "#{tempip}"
        $chap_user = "#{temp_user_name}"
      else

        line = line.to_s.strip
        iqnum = "#{iqnum}#{line}"
        $chap_user = "#{$chap_user}#{line}"

      end
      if (chap_user_list.include?($chap_user))
        chap_user_list.delete($chap_user)
      end
    end
    return chap_user_list
  end
  def verify_volaccess_iqn(iqnidList, volname)
	iqn_list = iqnidList
    if(iqn_list.is_a?(String))
		iqn_list = iqn_list.split(",")
    end	
	iqnlist = checkiqn(iqn_list, volname)
    if (iqnlist.size == 0)
	   #return true
	   return false
	else
	   #return false
	   return true
	end
  end
  def checkiqn(iqnList, volname)
	$transport = url_parse($device_conf['url'])
    response = ` python "#{$pythonFilePath}" #{$transport.host} #{$transport.user} #{$transport.password} volumeaccessshow #{volname}`
    if (response.match("% Error"))
      return "Unable to retrieve the iqn access for the volume #{volname} because of the following exception: #{response}."
    end
    counter = 0
    response.split("\n").each do |line|
      if (counter < 3)
        counter = counter + 1
        next
      end
      if (line.to_s.strip.length == 0)
        next
      end
      res = line.to_s.strip.split(" ")
      tempid = res[0]
      iq = res[1]
      tempip = res[2]
      if (tempid.to_s.strip =~ /^\d+$/ )
        $iqnum = "#{iq}"
        $id = "#{tempid}"
        $ip = "#{tempip}"
      else
        line = line.to_s.strip
        $iqnum = "#{$iqnum}#{line}"
      end
      if (iqnList.include?($iqnum))
        iqnList.delete($iqnum)
      elsif (iqnList.include?($ip))
        iqnList.delete($ip)
      end
    end
    return iqnList
  end
  def verify_volume_clone(volname)
	$transport = url_parse($device_conf['url'])
    response = `python "#{$pythonFilePath}" #{$transport.host} #{$transport.user} #{$transport.password} volumeshow #{volname}`
	if (response.match("Error"))
      return "Failed to retrieve volume information.#{response}"
	else
	  return true
    end
    #return response
  end
  def verify_volume_clone_property(volproperty, propertyvalue, volname)
	$transport = url_parse($device_conf['url'])
    response = `python "#{$pythonFilePath}" #{$transport.host} #{$transport.user} #{$transport.password} volumeshow #{volname}`
	if (response.match("Error"))
      return "Failed to retrieve volume information.#{response}"
	else
		volume_info = []
		volume_size = ""
		volume_name = ""
		volume_access = ""
		volume_template = ""
		volume_snap = ""
		volume_conn = ""
		volume_permission = ""
		volume_status = ""
		volume_thinprovision = ""
		volume_snapreserve = ""
		volume_thinminreserve = ""
		volume_thingrowthwarn = ""
		volume_thingrowthmax = ""
		#volume_thinwarnsoftthres = ""
		#volume_thinwarnhardthres = ""
		response.split("\n").each do |line|
		  res = line.split(":")
		  name = res[0].to_s.strip
		  value = res[1].to_s.strip
		  volume_status = get_status(name,value,volume_status)
		  volume_size = get_size(name,value,volume_size)
		  volume_name = get_name(name,value,volume_name)
		  volume_access = get_access(name,value,volume_access)
		  volume_template = get_template(name,value,volume_template)
		  volume_snap = get_snap(name,value,volume_snap)
		  volume_conn = get_conn(name,value,volume_conn)
		  volume_permission = get_permission(name,value,volume_permission)
		  volume_thinprovision = get_thinprovision(name,value,volume_thinprovision)
		  volume_snapreserve = get_snapreserve(name,value,volume_snapreserve)
		  volume_thinminreserve	= get_thinminreserve(name,value,volume_thinminreserve)
		  volume_thingrowthwarn	= get_thingrowthwarn(name,value,volume_thingrowthwarn)
		  volume_thingrowthmax	= get_thingrowthmax(name,value,volume_thingrowthmax)
		  #volume_thinwarnsoftthres	= get_thinwarnsoftthres(name,value,volume_thinwarnsoftthres)
		  #volume_thinwarnhardthres	= get_thinwarnhardthres(name,value,volume_thinwarnhardthres)
		  
		end
		
		if ( volproperty == 'state')
			if (volume_status.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'initsize')
			if (volume_size.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'volname')
			if (volume_name.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'multihostaccess')
			if (volume_access.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'template')
			if (volume_template.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'snap')
			if (volume_snap.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'connections')
			if (volume_conn.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'permissions')
			if (volume_permission.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'thinprovision')
			if (volume_thinprovision.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'snapreserve')
			if (volume_snapreserve.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'thinminreserve')
			if (volume_thinminreserve.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'thingrowthwarn')
			if (volume_thingrowthwarn.match(propertyvalue))
				return true
			end
		elsif (volproperty == 'thingrowthmax')
			if (volume_thingrowthmax.match(propertyvalue))
				return true
			end
		else
			return false
		end
	end
			
  end
  def verify_get_iqn(volname)
	$transport = url_parse($device_conf['url'])
    response = ` python "#{$pythonFilePath}" #{$transport.host} #{$transport.user} #{$transport.password} volumeshow #{volname}`
    if (response.match("Error"))
      return "Failed to show IQN Info to the Volume  #{volname} due the following: #{response}"
	  #return false
	else
		response.split("\n").each do |line|
		  if(line.to_s.include?"iSCSI Name:")
			array = line.split(": ")
			$iqn_name = array[1]
		  else
			next
		  end
		end
		if ($iqn_name != '')
			return true
		else
			return false
		end
	end
  end
  def verify_chapuser(chapuser)
	$transport = url_parse($device_conf['url'])
    response = ` python "#{$pythonFilePath}" #{$transport.host} #{$transport.user} #{$transport.password} "show chapuser"`
    if (response.match("Error"))
      return "Failed to get chap user  #{chapuser} info  due the following: #{response}"
	  #return false
	else
		chap_user_info = []
		counter = 0
		volcount = 1
		response.split("\n").each do |line|
		  if (counter < 3)
			counter = counter + 1
			next
		  else
			# Pull out relevant info
			array = line.split(' ')
			chap_user_name = array[0]
			# Construct hash
			chap_info = { :name => chap_user_name}
			# Add to array
			chap_user_info << chap_info
			volcount = volcount + 1
		  end
		end
		chap_info = chap_user_info
		chap_info.each do |chap_user|
		  # Resource existence validation
		  # Comparing if resource exists or not at equallogic...
		  if("#{chapuser}" == chap_user[:name])
			$is_new_vol_exist = "true"
		  end
		end
		if($is_new_vol_exist == "true")
		  return true
		else
		  return false
		end
	end
  end
  
  
  
  def get_status(name,value,volume_status)
    if name == "Status"
      volume_status = value
    end
    volume_status
  end

  def get_size(name,value,volume_size)
    if name == "Size"
      volume_size = value
    end
    volume_size
  end

  def get_name(name,value,volume_name)
    if name == "Name"
      volume_name = value
    end
    volume_name
  end

  def get_access(name,value,volume_access)
    if name == "MultiHostAccess"
      volume_access = value
    end
    volume_access
  end

  def get_template(name,value,volume_template)
    if name == "Template"
      volume_template = value
    end
    volume_template
  end

  def get_snap(name,value,volume_snap)
    if name == "Snapshots"
      volume_snap = value
    end
    volume_snap
  end

  def get_conn(name,value,volume_conn)
    if name == "Connections"
      volume_conn = value
    end
    volume_conn
  end

  def get_permission(name,value,volume_permission)
    if name == "Permission"
      volume_permission = value
    end
    volume_permission
  end
  
  def get_thinprovision(name,value,volume_thinprovision)
    if name == "Thin-Provision"
      volume_thinprovision = value
    end
    volume_thinprovision
  end

  def get_snapreserve(name,value,volume_snapreserve)
    if name == "Snap-Reserve"
      volume_snapreserve = value
    end
    volume_snapreserve
  end

  def get_thinminreserve(name,value,volume_thinminreserve)
    if name == "Thin-Min-Reserve"
      volume_thinminreserve = value
    end
    volume_thinminreserve
  end

  def get_thingrowthwarn(name,value,volume_thingrowthwarn)
    if name == "Thin-Growth-Warn"
      volume_thingrowthwarn = value
    end
    volume_thingrowthwarn
  end

  def get_thingrowthmax(name,value,volume_thingrowthmax)
    if name == "Thin-Growth-Max"
      volume_thingrowthmax = value
    end
    volume_thingrowthmax
  end

  def get_thinwarnsoftthres(name,value,volume_thinwarnsoftthres)
    if name == "Thin-Warn-Soft-Thres"
      volume_thinwarnsoftthres = value
    end
    volume_thinwarnsoftthres
  end

  def get_thinwarnhardthres(name,value,volume_thinwarnhardthres)
    if name == "Thin-Warn-Hard-Thres"
      volume_thinwarnhardthres = value
    end
    volume_thinwarnhardthres
  end
  class Transport_urlparse
	attr_accessor :host, :user, :password
	def initialize
	end
  end
		  
end

