require 'spec_helper'
require 'puppet/provider/equallogic'

describe Puppet::Provider::Equallogic do

  let(:equallogic_prov_obj) { described_class.new }

  describe "transport method" do
	
	before :each do
		#Puppet::Util::Log.level = :debug
		#Puppet::Util::Log.newdestination(:console)
	end
  
    it "with uninitialized device and no url should return error - context1 (:device)" do
		# the Equallogic Device expects a valid url path - unintialized device.
		equallogic_prov_obj.stubs(:transport).returns(Puppet::Error)
		expect { equallogic_prov_obj.device }.to raise_error(Puppet::Error)
    end

	it "with uninitialized device and no url should return error - context2 (:transport)" do
		# the Equallogic Device expects a valid url path - unintialized device.
		equallogic_prov_obj.stubs(:transport).returns(Puppet::Error)
		expect { equallogic_prov_obj.transport }.to raise_error
    end
	
    it "with uninitialized device and a unresolvable url should return error - context1 (invalid url)" do
	
      # the Equallogic Device expects a valid url path.
      Facter.stubs(:value).with(:url).returns('volume.example.com')
	  deviceobj = equallogic_prov_obj.device
	  eql_transport = equallogic_prov_obj.transport
	  equallogic_prov_obj.stubs(:transport).returns("Error")
	  expect{expect(eql_transport).to eq(eql_transport.match("Error"))}.to raise_error(Puppet::Error)
	  
    end
	
	it "with uninitialized device and a unresolvable url should return error - context2 (no password)" do
	
      # the Equallogic Device expects a valid url path.
      Facter.stubs(:value).with(:url).returns('https://eqluser@192.168.1.100')
	  deviceobj = equallogic_prov_obj.device
	  eql_transport = equallogic_prov_obj.transport
	  equallogic_prov_obj.stubs(:transport).returns("Error")
	  expect{expect(eql_transport).to eq(eql_transport.match("Error"))}.to raise_error(Puppet::Error)
	  
    end

	it "with uninitialized device and a unresolvable url should return error - context4 (http)" do
	
      # the Equallogic Device expects a valid url path.
      Facter.stubs(:value).with(:url).returns('http://eqluser:equluser@192.168.1.100')
	  deviceobj = equallogic_prov_obj.device
	  eql_transport = equallogic_prov_obj.transport
	  equallogic_prov_obj.stubs(:transport).returns("Error")
	  expect{expect(eql_transport).to eq(eql_transport.match("Error"))}.to raise_error(Puppet::Error)
	  
    end
	
	it "with initialized device and valid url scenario" do
	
	  Facter.stubs(:value).with(:url).returns('https://eqluser:equluser@192.168.1.100')
	  deviceobj = equallogic_prov_obj.device
	  eql_transport = equallogic_prov_obj.transport
	  equallogic_prov_obj.stubs(:transport).returns("Error")
	  expect{expect(eql_transport).to eq(eql_transport.match("Error"))}.to raise_error(Puppet::Error)
    end
	
  end
end

