#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'fixtures/transport/transport_mock'

describe Puppet::Type.type(:equallogic_volume_access_chap).provider(:equallogic_volume_access_chap) do
	
	before :each do
		@transport = Transport_mock.new.transport
		described_class.stubs(:suitable?).returns true
		Puppet::Type.type(:equallogic_volume_access_chap).stubs(:defaultprovider).returns described_class
	end

	let :create_volume_access_chap do
		Puppet::Type.type(:equallogic_volume_access_chap).new(
			:name          		=> 'NewVolume1',
			:ensure        		=> 'present',
			:chap_user_name 	=> 'chap_user_name',
		)
	end

	let :destroy_volume_access_chap do
		Puppet::Type.type(:equallogic_volume_access_chap).new(
			:name          		=> 'NewVolume1',
			:ensure        		=> 'absent',
			:chap_user_name 	=> 'chap_user_name',
		)
	end

	let :provider do
		described_class.new( )
	end

	describe "when asking exists?" do
		it "should return true if resource/volume is present" do
			create_volume_access_chap.provider.stubs(:transport).returns @transport
			create_volume_access_chap.provider.set(:name => 'NewVolume1', :ensure => :present)
			create_volume_access_chap.provider.should_not be_exists
		end
		it "should return false if resource/volume is absent" do
			destroy_volume_access_chap.provider.stubs(:transport).returns @transport
			destroy_volume_access_chap.provider.set(:name => 'NewVolume1', :ensure => :absent)
			destroy_volume_access_chap.provider.should_not be_exists
		end
	end

	describe "when creating a new chap access for volume" do
		it "should be able to create new chap access with a volume" do
			create_volume_access_chap.provider.stubs(:transport).returns @transport
			create_volume_access_chap.provider.stubs(:create_volume_chap_access).returns("Access control record created with ID")
			create_volume_access_chap.provider.create
		end
		it "should be able to create new chap access with a volume" do
			create_volume_access_chap.provider.stubs(:transport).returns @transport
			create_volume_access_chap.provider.expects(:create_volume_chap_access).once.returns("% Error")
			expect {create_volume_access_chap.provider.create}.to raise_error(Puppet::Error)
		end
	end

	describe "when destroying a chap access for volume" do
		it "should be able to destroy a chap access with volume with valid values" do
			destroy_volume_access_chap.provider.stubs(:transport).returns @transport
			destroy_volume_access_chap.provider.stubs(:delete_accessid_list).once.returns('1')
			destroy_volume_access_chap.provider.destroy
		end
		it "should be able to destroy a chap access with volume with valid values" do
			destroy_volume_access_chap.provider.stubs(:transport).returns @transport
			destroy_volume_access_chap.provider.expects(:show_volume_access).once.returns("Error logging")
			expect {destroy_volume_access_chap.provider.destroy}.to raise_error(Puppet::Error)
		end
		it "should be able to destroy a chap access with volume with valid values" do
			destroy_volume_access_chap.provider.stubs(:transport).returns @transport
			destroy_volume_access_chap.provider.expects(:show_volume_access).once.returns("% Error")
			expect {destroy_volume_access_chap.provider.destroy}.to raise_error(Puppet::Error)
		end
	end
	
	describe "when flushing a property hash" do
		it "should flush property hash, where ensure is present" do
			create_volume_access_chap.provider.stubs(:transport).returns @transport
			create_volume_access_chap.provider.set(:ensure => :present)
			create_volume_access_chap.provider.flush
		end
		it "should flush property hash, where ensure is absent" do
			destroy_volume_access_chap.provider.stubs(:transport).returns @transport
			destroy_volume_access_chap.provider.set(:ensure => :absent)
			destroy_volume_access_chap.provider.stubs(:flush).returns(true)
		end
	end
end
