#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'fixtures/transport/transport_mock'

describe Puppet::Type.type(:equallogic_volume_access_iqn).provider(:equallogic_volume_access_iqn) do

	before :each do
		@transport = Transport_mock.new.transport
		described_class.stubs(:suitable?).returns true
		Puppet::Type.type(:equallogic_volume_access_iqn).stubs(:defaultprovider).returns described_class
	end
	  
	let :create_volume_access_iqn do
		Puppet::Type.type(:equallogic_volume_access_iqn).new(
			:name          		=> 'NewVolume1',
			:ensure        		=> 'present',
			:iqnorip       		=> ['1.2.2.2'],
		)
	end

	let :destroy_volume_access_iqn do
		Puppet::Type.type(:equallogic_volume_access_iqn).new(
			:name          		=> 'NewVolume1',
			:ensure        		=> 'absent',
			:iqnorip       		=> ['1.2.2.2'],
		)
	end

	let :provider do
		described_class.new( )
	end

	describe "when asking exists?" do
		it "should return true if resource/volume is present" do
			create_volume_access_iqn.provider.stubs(:transport).returns @transport
			create_volume_access_iqn.provider.set(:ensure => :present)
			create_volume_access_iqn.provider.should_not be_exists
		end

		it "should return false if resource/volume is absent" do
			destroy_volume_access_iqn.provider.stubs(:transport).returns @transport
			destroy_volume_access_iqn.provider.set(:ensure => :absent)
			destroy_volume_access_iqn.provider.should_not be_exists
		end
	end


	describe "when creating a new iqn access for volume" do
		it "should be able to create a iqn access with volume" do
			create_volume_access_iqn.provider.stubs(:transport).returns @transport
			create_volume_access_iqn.provider.stubs(:create_vol_iqn_access).returns("Access control record created with ID")
			create_volume_access_iqn.provider.create
		end
		it "should not be able to create new iqn access with a volume" do
			create_volume_access_iqn.provider.stubs(:transport).returns @transport
			create_volume_access_iqn.provider.expects(:create_vol_iqn_access).once.returns("% Error")
			expect {create_volume_access_iqn.provider.create}.to raise_error(Puppet::Error)
		end
	end

	describe "when destroying a iqn access with volume" do
		it "should be able to destroy a iqn access with volume with valid values" do
			destroy_volume_access_iqn.provider.stubs(:transport).returns @transport
			destroy_volume_access_iqn.provider.stubs(:delete_accessid_list).once.returns('1')
			destroy_volume_access_iqn.provider.destroy
		end
		it "should not be able to destroy a iqn access with volume" do
			destroy_volume_access_iqn.provider.stubs(:transport).returns @transport
			destroy_volume_access_iqn.provider.expects(:show_volume_access).once.returns("Error logging")
			expect {destroy_volume_access_iqn.provider.destroy}.to raise_error(Puppet::Error)
		end
		it "should not be able to destroy a iqn access with volume" do
			destroy_volume_access_iqn.provider.stubs(:transport).returns @transport
			destroy_volume_access_iqn.provider.expects(:show_volume_access).once.returns("% Error")
			expect {destroy_volume_access_iqn.provider.destroy}.to raise_error(Puppet::Error)
		end
	end
	  
	describe "when flushing a property hash" do
		it "should flush property hash, where ensure is present" do
			create_volume_access_iqn.provider.stubs(:transport).returns @transport
			create_volume_access_iqn.provider.set(:ensure => :present)
			create_volume_access_iqn.provider.flush
		end
		it "should flush property hash, where ensure is absent" do
			destroy_volume_access_iqn.provider.stubs(:transport).returns @transport
			destroy_volume_access_iqn.provider.set(:ensure => :absent)
			destroy_volume_access_iqn.provider.stubs(:flush).returns(true)
		end
	end
	
end


