#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'fixtures/transport/transport_mock'

describe Puppet::Type.type(:equallogic_volume_chap_user).provider(:equallogic_volume_chap_user) do

	before :each do
		@transport = Transport_mock.new.transport
		described_class.stubs(:suitable?).returns true
		Puppet::Type.type(:equallogic_volume_chap_user).stubs(:defaultprovider).returns described_class
	end

	let :create_chap_user do
		Puppet::Type.type(:equallogic_volume_chap_user).new(
			:name          		=> 'ChapUser1',
			:ensure        		=> 'present',
			:chap_password   	=> 'chapuserpasswd1',
			:state				=> 'enable',
		)
	end

	let :destroy_chap_user do
		Puppet::Type.type(:equallogic_volume_chap_user).new(
			:name          		=> 'ChapUser1',
			:ensure        		=> 'absent',
			:chap_password   	=> 'chapuserpasswd1',
			:state				=> 'enable',
		)
	end

	let :provider do
		described_class.new( )
	end

	describe "when asking exists?" do
		it "should return true if new chap user is present" do
			create_chap_user.provider.stubs(:transport).returns @transport
			create_chap_user.provider.stubs(:get_chap_user_info).returns( [{:name => 'ChapUser1'}])
			create_chap_user.provider.set(:ensure => :present)
			create_chap_user.provider.should be_exists
		end
		it "should return true if new chap user is absent" do
			destroy_chap_user.provider.stubs(:transport).returns @transport
			destroy_chap_user.provider.stubs(:get_chap_user_info).returns( [{:name => 'ChapUser1'}])
			destroy_chap_user.provider.set(:ensure => :absent)
			destroy_chap_user.provider.should be_exists
		end
	end

	describe "when creating a new chap user" do
		it "should be able to create a new chap user" do
			create_chap_user.provider.stubs(:transport).returns @transport
			create_chap_user.provider.stubs(:create_chap_user).returns("CHAP user creation succeeded.")
			create_chap_user.provider.create
		end
		it "should not able to create a new chap user" do
			create_chap_user.provider.stubs(:transport).returns @transport
			create_chap_user.provider.expects(:create_chap_user).once.returns("% Error")
			expect {create_chap_user.provider.create}.to raise_error(Puppet::Error)
		end	
	end

	describe "when destroying a chap user" do
		it "should be able to destroy a chap user" do
			destroy_chap_user.provider.stubs(:transport).returns @transport
			destroy_chap_user.provider.stubs(:delete_chap_user).returns("CHAP user deletion succeeded.")
			destroy_chap_user.provider.destroy
		end
		it "should not able to destroy a chap user" do
			destroy_chap_user.provider.stubs(:transport).returns @transport
			destroy_chap_user.provider.expects(:delete_chap_user).once.returns("% Error")
			expect {destroy_chap_user.provider.destroy}.to raise_error(Puppet::Error)
		end
	end

	describe "when flushing a property hash" do
		it "should flush property hash, where ensure is present" do
			create_chap_user.provider.stubs(:transport).returns @transport
			create_chap_user.provider.set(:ensure => :present)
			create_chap_user.provider.flush
		end
		it "should flush property hash, where ensure is absent" do
			destroy_chap_user.provider.stubs(:transport).returns @transport
			destroy_chap_user.provider.set(:ensure => :absent)
			destroy_chap_user.provider.stubs(:flush).returns(true)
		end
	end
	
end
