#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'fixtures/transport/transport_mock'

describe Puppet::Type.type(:equallogic_volume_clone).provider(:equallogic_volume_clone) do

    before :each do
        @transport = Transport_mock.new.transport
        described_class.stubs(:suitable?).returns true
        Puppet::Type.type(:equallogic_volume_clone).stubs(:defaultprovider).returns described_class
        create_volume_clone.provider.stubs(:transport).returns @transport
    end
      
    let :create_volume_clone do
        Puppet::Type.type(:equallogic_volume_clone).new(
          :name             => 'NewVolume1',
          :ensure           => 'present',
          :volume_name      => 'CloneVolume1',
        )
    end
  
    let :provider do
        described_class.new()
    end
  
	context "when create clone for a given volume is run  " do
		it "should have create method defined for create clone from volume" do
			create_volume_clone.provider.class.instance_method(:create).should_not == nil
		end
		it "should have destroy method defined for create clone from volume" do
			create_volume_clone.provider.class.instance_method(:destroy).should_not == nil
		end
		it "should have exists? method defined for create clone from volume" do
			create_volume_clone.provider.class.instance_method(:exists?).should_not == nil
		end
	end

	describe "when asking exists?" do
		it "should return true if resource/volume is present" do
			create_volume_clone.provider.stubs(:exists?).once.returns(true)
			create_volume_clone.provider.exists?.should eq(true)
		end
		it "should return false if resource/volume is absent" do
			create_volume_clone.provider.stubs(:exists?).once.returns(false)
			create_volume_clone.provider.exists?.should eq(false)      
		end
	end

	describe "when creating clone if Invalid credentials/volume name" do
		it "for create clone for equallogic volume." do
			create_volume_clone.provider.expects(:create_volume_clone).once.returns("Error: Only ASCII")
			expect {create_volume_clone.provider.create}.to raise_error(Puppet::Error)
		end
	end

	describe "when creating clone if Not correct Parameters" do
		it "for create clone for equallogic volume." do
			create_volume_clone.provider.expects(:create_volume_clone).once.returns("% Error")
			expect {create_volume_clone.provider.create}.to raise_error(Puppet::Error)
		end
	end

	describe "when calling destroy " do
		it "when creating clone equallogic IQN volume with ensure == absent." do
			create_volume_clone.provider.stubs(:destroy).returns(:absent)
			create_volume_clone.provider.destroy.should eq(:absent)        
		end
	end
	
end


