#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'fixtures/transport/transport_mock'


describe Puppet::Type.type(:equallogic_volume_get_iqn).provider(:equallogic_volume_get_iqn) do

	before :each do
		@transport = Transport_mock.new.transport
		described_class.stubs(:suitable?).returns true
		Puppet::Type.type(:equallogic_volume_get_iqn).stubs(:defaultprovider).returns described_class
		get_iqn_volume.provider.stubs(:transport).returns @transport
	end
	
	let :get_iqn_volume do
		Puppet::Type.type(:equallogic_volume_get_iqn).new(
			:name          		=> 'NewVolume1',
			:ensure        		=> 'present',
		)
	end

	let :provider do
		described_class.new()
	end
	
	context "when get IQN for a given volume is run  " do
		it "should have create method defined for get IQN from volume" do
			get_iqn_volume.provider.class.instance_method(:create).should_not == nil
		end
		it "should have destroy method defined for  get IQN from volume" do
			get_iqn_volume.provider.class.instance_method(:destroy).should_not == nil
		end
		it "should have exists? method defined for  get IQN from volume" do
			get_iqn_volume.provider.class.instance_method(:exists?).should_not == nil
		end
	end

	describe "when asking exists?" do
		it "should return true if resource/volume is present" do
			get_iqn_volume.provider.stubs(:exists?).once.returns(true)
			get_iqn_volume.provider.exists?.should eq(true)
		end
		it "should return false if resource/volume is absent" do
			get_iqn_volume.provider.stubs(:exists?).once.returns(false)
			get_iqn_volume.provider.exists?.should eq(false)      
		end
	end

	describe "when getting if Invalid credentials" do
		it "for get IQN equallogic volume." do
			get_iqn_volume.provider.expects(:get_iqn_of_volume).once.returns("Error logging")
			expect {get_iqn_volume.provider.create}.to raise_error(Puppet::Error)
		end
	end

	describe "when getting if Not correct Parameters" do
		it "for get IQN equallogic volume." do
			get_iqn_volume.provider.expects(:get_iqn_of_volume).once.returns("% Error")
			expect {get_iqn_volume.provider.create}.to raise_error(Puppet::Error)
		end
	end

	describe "when calling destroy " do
		it "when getting equallogic IQN volume with ensure == absent." do
			get_iqn_volume.provider.stubs(:destroy).returns(:absent)
			get_iqn_volume.provider.destroy.should eq(:absent)        
		end
	end

	describe "when flush" do
		it "get iqn flush should not be successful when ensure is present" do
			get_iqn_volume.provider.set(:ensure => :present)
			get_iqn_volume.provider.flush
		end
		it "get iqn flush should not be successful when ensure is absent" do
			get_iqn_volume.provider.set(:ensure => :absent)
			get_iqn_volume.provider.flush
		end
	end
	
end


