#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'

describe Puppet::Type.type(:equallogic_volume_access_iqn) do

	let :resource do
		described_class.new(
			:name          		=> 'NewVolume1',
			:ensure        		=> 'present',
			:iqnorip          	=> 'iqn.2001-05.com.equallogic:0-1ccc1-c4-79-qag',
		)
	end

	it "should have name as its keyattribute" do
		described_class.key_attributes.should == [:name]
	end

	describe "when validating attributes" do
		[:name, :iqnorip].each do |param|
			it "should hava a #{param} parameter" do
				described_class.attrtype(param).should == :param
			end
		end
		[:ensure].each do |property|
			it "should have a #{property} property" do
				described_class.attrtype(property).should == :property
			end
		end
	end

	describe "when validating values" do
		describe "for name" do
			it "should allow a valid volumename with ensure present" do
				described_class.new(:name => 'newvolOne', :ensure => 'present')[:name].should == 'newvolOne'
			end
			it "should allow a valid volumename with ensure absent" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent')[:name].should == 'newvolTwo'
			end
		end
		describe "for ensure" do
			it "should allow present" do
				described_class.new(:name => 'newvolOne', :ensure => 'present')[:ensure].should == :present
			end
			it "should allow absent" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent')[:ensure].should == :absent
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'foo') }.to raise_error Puppet::ResourceError
			end
		end
		describe "for validation of iqnorip" do
			it "should allow present" do
				described_class.new(:name => 'newvolOne', :iqnorip => 'iqn.2001-05.com.equallogic:0-1ccc1-c4-79-qag')[:iqnorip].should == 'iqn.2001-05.com.equallogic:0-1ccc1-c4-79-qag'
			end
			it "should allow absent" do
				described_class.new(:name => 'newvolTwo', :iqnorip => 'iqn.2-05.c.e:0-1c-c4-9-a')[:iqnorip].should == 'iqn.2-05.c.e:0-1c-c4-9-a'
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :iqnorip => 'abc.iqn.add.new.001') }.to raise_error Puppet::ResourceError
			end
		end
	end	
end