#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'

describe Puppet::Type.type(:equallogic_volume_chap_user) do

	let :resource do
		described_class.new(
			:name          		=> 'chapUserNew1',
			:ensure				=> 'present',
			:chap_password 		=> 'chapPassw0rd1',
			:state				=> 'enable',
		)
	end
 
	it "should have name as its keyattribute" do
		described_class.key_attributes.should == [:name]
	end
 
	describe "when validating attributes" do
		[:name, :chap_password,:state].each do |param|
			it "should hava a #{param} parameter" do
				described_class.attrtype(param).should == :param
			end
		end
		[:ensure].each do |property|
			it "should have a #{property} property" do
				described_class.attrtype(property).should == :property
			end
		end
	end

	describe "when validating values" do
		describe "for name" do
			it "should allow a valid chap user" do
				described_class.new(:name => 'newChapUser1')[:name].should == 'newChapUser1'
			end
			it "should allow a valid chap user" do
				described_class.new(:name => 'newChapUser2')[:name].should == 'newChapUser2'
			end
		end

		describe "for chapuser password" do
			it "should allow present" do
				described_class.new(:name => 'newchapUser1', :chap_password => 'password1')[:chap_password].should == 'password1'
			end
			it "should allow absent" do
				described_class.new(:name => 'newchapUser2', :chap_password => 'newpassword99')[:chap_password].should == 'newpassword99'
			end
			it "should not allow invalid passwords " do
				expect { described_class.new(:name => 'newchapUser3', :chap_password => '!!@@##') }.to raise_error Puppet::ResourceError
			end
			it "should not allow empty passwords " do
				expect { described_class.new(:name => 'newchapUser3', :chap_password => '') }.to raise_error Puppet::ResourceError
			end
			it "should not allow 64 character passwords " do
				expect { described_class.new(:name => 'newchapUser3', :chap_password => 'abcdefghijklmnopqurstuvwxyzabcdefghijklmnopqurstuvwxyzabcdefghijklmnopqurstuvwxyz') }.to raise_error Puppet::ResourceError
			end
		end
            
		describe "for state" do
			it "should allow online" do
				described_class.new(:name => 'newchapOne', :state => 'enable')[:state].should == 'enable'
			end
			it "should allow offline" do
				described_class.new(:name => 'newchapTwo', :state => 'disable')[:state].should == 'disable'
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newchapThree', :state => '!!@@##') }.to raise_error Puppet::ResourceError
			end
		end 
	end	
end