#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'

describe Puppet::Type.type(:equallogic_volume_clone) do

	let :resource do
		described_class.new(
			:name          		=> 'NewVolumeClone1',
			:ensure        		=> 'present',
			:volume_name		=> 'NewVolume1',
		)
	end
  
	it "should have name as its keyattribute" do
		described_class.key_attributes.should == [:name]
	end

	describe "when validating attributes" do
		[:name, :volume_name].each do |param|
			it "should hava a #{param} parameter" do
				described_class.attrtype(param).should == :param
			end
		end
		[:ensure].each do |property|
			it "should have a #{property} property" do
				described_class.attrtype(property).should == :property
			end
		end
	end

	describe "when validating values" do
		describe "for volume name validation" do
			it "should allow a valid volumename of string" do
				described_class.new(:name => 'newvolOneKK',:volume_name => 'newvolOne')[:volume_name].should == 'newvolOne'
			end
			it "should allow a valid volume name" do
				described_class.new(:name => 'newvolOneKK',:volume_name => 'newvolTwo')[:volume_name].should == 'newvolTwo'
			end
		end
		describe "for ensure" do
			it "should allow present" do
				described_class.new(:name => 'newvolOne', :ensure => 'present')[:ensure].should == :present
			end
			it "should allow absent" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent')[:ensure].should == :absent
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'foo') }.to raise_error Puppet::ResourceError
			end
		end
		describe "for name" do
			it "should allow a valid volume name of string" do
				described_class.new(:name => 'newvolOne')[:name].should == 'newvolOne'
			end
			it "should allow a valid volume name with numerics" do
				described_class.new(:name => 'newvolTwo123')[:name].should == 'newvolTwo123'
			end
		end
	end	
end
