#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'

describe Puppet::Type.type(:equallogic_volume_get_iqn) do

	let :resource do
		described_class.new(
			:name          		=> 'NewVolume1',
			:ensure        		=> 'present',
		)
	end

	it "should have name as its keyattribute" do
		described_class.key_attributes.should == [:name]
	end

	describe "when validating attributes" do
		[:name].each do |param|
			it "should hava a #{param} parameter" do
				described_class.attrtype(param).should == :param
			end
		end
		[:ensure].each do |property|
			it "should have a #{property} property" do
				described_class.attrtype(property).should == :property
			end
		end
	end

	describe "when validating values" do
		describe "for name" do
			it "should allow a valid volumename where ensure present" do
				described_class.new(:name => 'newvolOne', :ensure => 'present')[:name].should == 'newvolOne'
			end
			it "should allow a valid volumename where ensure absent" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent')[:name].should == 'newvolTwo'
			end
		end
		describe "for ensure" do
			it "should allow present" do
				described_class.new(:name => 'newvolOne', :ensure => 'present')[:ensure].should == :present
			end
			it "should allow absent" do
				described_class.new(:name => 'newvolTwo', :ensure => 'absent')[:ensure].should == :absent
			end
			it "should not allow something else" do
				expect { described_class.new(:name => 'newvolThree', :ensure => 'foo') }.to raise_error Puppet::ResourceError
			end
		end
	end	
end