#! /usr/bin/env ruby

require 'spec_helper'
require 'yaml'
require 'fixtures/transport/transport_mock'
require 'puppet/util/network_device/equallogic'
require 'puppet/util/network_device/equallogic/facts'
require 'json'

describe Puppet::Util::NetworkDevice::Equallogic::Facts do

	let :transport do
		@transport = Transport_mock.new.transport
		@transport
	end

	let :factsobj do
		described_class.new(transport)
	end
  
	before :each do
		@mockmap = {}
		@facts = {}
	end

	describe "facts from getgeneralsettinginfo" do
		it "when given valid values - execute" do
			factsobj.stubs(:getgrpparamresponse).once.returns("Mock Name: Mock Value")
			factsobj.getgeneralsettinginfo
		end
		it "when given expected values- group name" do
			@mockmap['Group Name'] = 'Mock Value'
			@facts = JSON.pretty_generate(@mockmap)
			factsobj.stubs(:getgrpparamresponse).returns("Name: Mock Value")
			factsobj.stubs(:getgeneralsettinginfo).returns(JSON.pretty_generate(@mockmap))
			factsobj.getgeneralsettinginfo.should eq(@facts) 
		end
		it "when given expected values - ip address" do
			@mockmap['IP Address'] = 'Mock Value'
			@facts = JSON.pretty_generate(@mockmap)
			factsobj.stubs(:getgrpparamresponse).returns("Group-Ipaddress: Mock Value")
			factsobj.stubs(:getgeneralsettinginfo).returns(JSON.pretty_generate(@mockmap))
			factsobj.getgeneralsettinginfo.should eq(@facts) 
		end
		it "when given expected values - location" do
			@mockmap['Location'] = 'Mock Value'
			@facts = JSON.pretty_generate(@mockmap)
			factsobj.stubs(:getgrpparamresponse).returns("Location: Mock Value")
			factsobj.stubs(:getgeneralsettinginfo).returns(JSON.pretty_generate(@mockmap))
			factsobj.getgeneralsettinginfo.should eq(@facts) 
		end
		it "when given invalid values - error" do
			factsobj.stubs(:getgrpparamresponse).once.returns("% Error")
			expect {factsobj.getgeneralsettinginfo}.to raise_error(Puppet::Error)
		end
	end

	describe "facts from getstoragepoolinfo" do
		it "when given valid values - execute" do
			factsobj.stubs(:getpoolresponse).once.returns("Mock Member Pool")
			factsobj.getstoragepoolinfo
		end
		it "when given expected values - equal to expected" do
			@facts = JSON.pretty_generate(@mockmap)
			factsobj.stubs(:getgrpparamresponse).returns({})
			factsobj.stubs(:getgeneralsettinginfo).returns(JSON.pretty_generate(@mockmap))
			factsobj.getgeneralsettinginfo.should eq(@facts) 
		end
		it "when given invalid values - error" do
			factsobj.stubs(:getpoolresponse).once.returns("% Error")
			expect {factsobj.getstoragepoolinfo}.to raise_error(Puppet::Error)
		end
	end

	describe "facts from getmembercountinfo" do
		it "when given valid values - execute" do
			memberCounter = rand(0..10)
			factsobj.stubs(:getmemberresponse).returns("Member Response")
			begin
				factsobj.getmembercountinfo
			rescue
				puts memberCounter
			end
		end
		it "when given valid values - equal to expected" do
			memberCounter = 5
			factsobj.stubs(:getmemberresponse).once.returns("Member Response")
			begin
				factsobj.getmembercountinfo.should eq(memberCounter)
			rescue
				puts memberCounter
			end
		end
		it "when given invalid values - error" do
			memberCounter = 5
			factsobj.stubs(:getmemberresponse).once.returns("% Error")
			expect {factsobj.getmembercountinfo}.to raise_error(Puppet::Error)
		end
		it "when given valid values - execute" do
			memberCounter = rand(0..10)
			factsobj.stubs(:getgrpparamresponse).returns("Member Response")
			factsobj.stubs(:getmemberinforesponse).returns("Member Info Response")
			begin
				factsobj.getmembercountinfo
			rescue
				puts memberCounter
			end
		end
		it "when given valid values - eqaul to expected" do
			memberCounter = rand(0..10)
			factsobj.stubs(:getmemberresponse).returns("Member Response")
			factsobj.stubs(:getmemberinforesponse).returns("Member Info Response")
			begin
				factsobj.getmembercountinfo.should eq(memberCounter)
			rescue
				puts memberCounter
			end
		end
		it "when given invalid values - error" do
			memberCounter = rand(0..10)
			factsobj.stubs(:getmemberresponse).once.returns("Member Response")
			factsobj.stubs(:getmemberinforesponse).once.returns("% Error")
			expect {factsobj.getmembercountinfo}.to raise_error(Puppet::Error)
		end
	end

	describe "facts from getcollectioninfo" do
		it "when given valid values - execute" do
			@facts = JSON.pretty_generate(@mockmap)
			factsobj.stubs(:getcollectioninforesponse).once.returns("Collection info response")
			factsobj.stubs(:getsnapcollectioninforesponse).once.returns("Snap collection info response")
			begin
				factsobj.getcollectioninfo
			rescue
				puts 'Get collection info'
			end
		end
		it "when given valid values - volume collection expected to equal" do
			@mockmap['Volume Collections'] = 'Mock Value'
			@facts = JSON.pretty_generate(@mockmap)
			factsobj.stubs(:getcollectioninforesponse).returns("Collection info response")
			factsobj.stubs(:getsnapcollectioninforesponse).returns("Snap collection info response")
			factsobj.stubs(:getcollectioninfo).returns(JSON.pretty_generate(@mockmap))
			factsobj.getcollectioninfo.should eq(@facts) 
		end
		it "when given valid values - custom snapshot expected to equal" do
			@mockmap['Custom Snapshot Collections'] = 'Mock Value'
			@facts = JSON.pretty_generate(@mockmap)
			factsobj.stubs(:getcollectioninforesponse).returns("Collection info response")
			factsobj.stubs(:getsnapcollectioninforesponse).returns("Snap collection info response")
			factsobj.stubs(:getcollectioninfo).returns(JSON.pretty_generate(@mockmap))
			factsobj.getcollectioninfo.should eq(@facts)
		end
		it "when given valid values - snapshot collection expected to equal" do
			@mockmap['Snapshot Collections'] = 'Mock Value'
			@facts = JSON.pretty_generate(@mockmap)
			factsobj.stubs(:getcollectioninforesponse).returns("Collection info response")
			factsobj.stubs(:getsnapcollectioninforesponse).returns("Snap collection info response")
			factsobj.stubs(:getcollectioninfo).returns(JSON.pretty_generate(@mockmap))
			factsobj.getcollectioninfo.should eq(@facts)
		end
	end

	describe "facts from getvolumereserve" do
		it "when given valid values - execute" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getvolumereserveresponse).once.returns(response)
			factsobj.getvolumereserve
		end
		it "when given valid values - expected to equal" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getvolumereserveresponse).once.returns(response)
			factsobj.getvolumereserve.should eq(volumesInUse)
		end
		it "when given invalid values - Failed" do
			volumesInUse = "N/A"
			factsobj.stubs(:getvolumereserveresponse).once.returns("Failed")
			factsobj.getvolumereserve.should eq(volumesInUse)
		end
		it "when given invalid values - No SNMP" do
			volumesInUse = "N/A"
			factsobj.stubs(:getvolumereserveresponse).once.returns("No SNMP")
			factsobj.getvolumereserve.should eq(volumesInUse)
		end
	end

	describe "facts from getfreespace" do
		it "when given valid values - execute" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getfreespaceresponse).once.returns(response)
			factsobj.getfreespace
		end
		it "when given valid values - expected to equal" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getfreespaceresponse).once.returns(response)
			factsobj.getfreespace.should eq(volumesInUse)
		end
		it "when given invalid values - Failed" do
			volumesInUse = "N/A"
			factsobj.stubs(:getfreespaceresponse).once.returns("Failed")
			factsobj.getfreespace.should eq(volumesInUse)
		end
		it "when given invalid values - No SNMP" do
			volumesInUse = "N/A"
			factsobj.stubs(:getfreespaceresponse).once.returns("No SNMP")
			factsobj.getfreespace.should eq(volumesInUse)
		end
	end
	
	describe "facts from getdelegated" do
		it "when given valid values - execute" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getdelegatedresponse).once.returns(response)
			factsobj.getdelegated
		end
		it "when given valid values - equal to expected" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getdelegatedresponse).once.returns(response)
			factsobj.getdelegated.should eq(volumesInUse)
		end
		it "when given invalid values - Failed" do
			volumesInUse = "N/A"
			factsobj.stubs(:getdelegatedresponse).once.returns("Failed")
			factsobj.getdelegated.should eq(volumesInUse)
		end
		it "when given invalid values - No SNMP" do
			volumesInUse = "N/A"
			factsobj.stubs(:getdelegatedresponse).once.returns("No SNMP")
			factsobj.getdelegated.should eq(volumesInUse)
		end
	end
	
	describe "facts from getreplicationreserve" do
		it "when given valid values - execute" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getreplicationreserveresponse).once.returns(response)
			factsobj.getreplicationreserve
		end
		it "when given valid values - equal to expected" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getreplicationreserveresponse).once.returns(response)
			factsobj.getreplicationreserve.should eq(volumesInUse)
		end
		it "when given invalid values - Failed case" do
			volumesInUse = "N/A"
			factsobj.stubs(:getreplicationreserveresponse).once.returns("Failed")
			factsobj.getreplicationreserve.should eq(volumesInUse)
		end
		it "when given invalid values - No SNMP" do
			volumesInUse = "N/A"
			factsobj.stubs(:getreplicationreserveresponse).once.returns("No SNMP")
			factsobj.getreplicationreserve.should eq(volumesInUse)
		end
	end
	
	describe "facts from getsnapshotreserve" do
		it "when given valid values - execute" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getsnapshotreserveresponse).once.returns(response)
			factsobj.getsnapshotreserve
		end
		it "when given valid values - equal to expected" do
			volumesInUse = "N/A"
			response = 'Volume in use reponse'
			volumesInUse = response.to_s.strip
			factsobj.stubs(:getsnapshotreserveresponse).once.returns(response)
			factsobj.getsnapshotreserve.should eq(volumesInUse)
		end
		it "when given invalid values - Failed case" do
			volumesInUse = "N/A"
			factsobj.stubs(:getsnapshotreserveresponse).once.returns("Failed")
			factsobj.getsnapshotreserve.should eq(volumesInUse)
		end
		it "when given invalid values - No SNMP" do
			volumesInUse = "N/A"
			factsobj.stubs(:getsnapshotreserveresponse).once.returns("No SNMP")
			factsobj.getsnapshotreserve.should eq(volumesInUse)
		end
	end

end
