#! /usr/bin/env ruby

require 'spec_helper'
require 'puppet/util/network_device/transport_equallogic'
require 'fixtures/transport/transport_mock'
require 'yaml'

describe Puppet::Util::NetworkDevice::Transport_equallogic do
  
  let :transport do
    @transport = Transport_mock.new.transport
    @transport
  end

  before :each do
    @transport_class = described_class.new
	@transport_class.host = "192.168.1.100"
	@transport_class.user = "eqladmin"
	@transport_class.password = "eqladmin"
	@transport_class.snmpcommunity = "snmpstring"
  end
 
  describe "#host" do
    it "returns the correct host" do
        @transport_class.host.should eql "192.168.1.100"
    end
  end
  
  describe "#user" do
	it "returns the correct user" do
		@transport_class.user.should eql "eqladmin"
	end
  end
  
  describe "#password" do
	it "returns the correct password" do
		@transport_class.password.should eql "eqladmin"
	end
  end
  
  describe "#snmpstring" do
	it "returns the correct snmpstring" do
		@transport_class.snmpcommunity.should eql "snmpstring"
	end
  end
    
  it "attribute host value, should match with assigned value" do
	  expect{expect(@transport_class.host).to eq("192.168.1.100")}.not_to raise_error
  end
  
  it "attribute user value, should match with assigned value" do
	  expect{expect(@transport_class.user).to eq("eqladmin")}.not_to raise_error
  end
  
  it "attribute password value, should match with assigned value" do
	  expect{expect(@transport_class.password).to eq("eqladmin")}.not_to raise_error
  end
  
  it "attribute snmpstring value, should match with assigned value" do
	  expect{expect(@transport_class.snmpcommunity).to eq("snmpstring")}.not_to raise_error
  end

  describe "When volume properties are requested" do
    it "valid request should return properties" do
      transport.stubs(:get_volumes).once.returns("Mock Volumes...")
      transport.get_volumes.should eq("Mock Volumes...")
    end
    it "invalid request should raise exception" do
      transport.stubs(:get_volumes).raises(Puppet::Error.new('SNMP Error'))
      expect {transport.get_volumes}.to raise_error(Puppet::Error)
    end
  end
  
end
